@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_ogla.erl 238 2008-01-27 14:05:47Z bjorng $
+%%     $Id$
 %%
 
 -module(wpc_ogla).
@@ -30,14 +30,14 @@ init() ->
 	{error,Reason} ->
 	    io:format("Failed to load ~s in ~s\n~s\n",
 		      [Name,Dir,erl_ddll:format_error(Reason)]),
-	    erlang:error(startup_fault)
+	    erlang:halt()
     end,
-    case open_port({spawn,Name},[]) of
-	Port when is_port(Port) ->
-	    register(wings_ogla_port, Port);
-	_ ->
-	    io:format("Failed to open port ~s\n", [Name]),
-	    erlang:error(startup_fault)
+    try
+	Port = open_port({spawn,Name},[]),
+	register(wings_ogla_port, Port)
+    catch error:_ ->
+	io:format("Failed to open port ~s.\n", [Name]),
+	erlang:halt()
     end,
 
     false.
