@@ -3,12 +3,12 @@
 %%
 %%     A semi-simple semi-automatic UV-mapping semi-plugin.
 %%
-%%  Copyright (c) 2002-2008 Dan Gudmundsson, Bjorn Gustavsson
+%%  Copyright (c) 2002-2009 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_autouv.erl 188 2008-01-01 15:05:07Z bjorng $
+%%     $Id$
 %%
 
 -module(wpc_autouv).
@@ -22,9 +22,7 @@
  
 -export([init/0,menu/2,command/2,redraw/1]).
 -export([handle_event/2,bg_image/0]). %% Debug
--import(lists, [sort/1,keysort/2,map/2,foldl/3,reverse/1,
-		append/1,delete/2,usort/1,max/1,min/1,
-		member/2,foreach/2,keysearch/3]).
+-import(lists, [sort/1,keysort/2,map/2,foldl/3,reverse/1,keysearch/3]).
 
 %% Exports to auv_seg_ui.
 -export([init_show_maps/4]).
@@ -58,18 +56,7 @@ auv_menu(help,_) ->
     {?__(2,"Generate UV mapping or texture"),
      ?__(25,"Re-segment object(s)"),
      ?__(3,"Force to segmenting mode (delete old segments)")};
-auv_menu(1,_What) -> 
-    case wings_pref:get_value(advanced_menus) of
-	false -> 
-	    [{?__(4,"Direct"), segment,
-	      ?__(5,"Open UV-window directly if selection already contains uv-coords")},
-	     {?__(8,"Force Segment (keep previous)"), segment_old,
-	      ?__(9,"Re-segment with old UV-coords ")},
-	     {?__(6,"Force Segment (delete previous)"), force_seg,
-	      ?__(7,"Delete old UV-coords and start over with segmenting")}];
-	true -> 
-	    {?MODULE, segment}
-    end;
+auv_menu(1,_What) -> {?MODULE, segment};
 auv_menu(2,_) -> {?MODULE, segment_old};
 auv_menu(3,_) -> {?MODULE, force_seg}.
 
@@ -263,7 +250,6 @@ create_uv_state(Charts, MatName, Fs, We, GeomSt = #st{shapes=Shs0},OrigSt) ->
 				 fun draw_background/1),
     wings_wm:menubar(Win, menubar()),
     wings_wm:send({menubar,Win}, {current_state,St}),
-    
     St.
 
 menubar() ->
@@ -280,9 +266,13 @@ menubar() ->
 	      ]
       end},
      {?__(8,"View"),view,
-      fun(_) ->
-	      [{?__(9,"Show/Hide Background Image"),toggle_background,
-		?__(10,"Toggle display of the background texture image")}]
+      fun(St) ->
+	      Menu0 = wings_view:menu(St),
+	      ShwBgImg = {?__(9,"Show/Hide Background Image"),toggle_background,
+	                  ?__(10,"Toggle display of the background texture image")},
+	      Menu = [I || I <- Menu0,
+			   keep_view_item(I)],
+	      [ShwBgImg|redundant_separators(Menu)]
       end},
      {?__(11,"Select"),select,
       fun(St) ->
@@ -292,6 +282,13 @@ menubar() ->
 	      redundant_separators(Menu)
       end}].
 
+
+keep_view_item(separator) -> true;
+keep_view_item({_,aim,_}) -> true;
+keep_view_item({_,highlight_aim,_}) -> true;
+keep_view_item({_,frame,_}) -> true;
+keep_view_item(_) -> false.
+
 keep_sel_item(separator) -> true;
 keep_sel_item({_,more,_}) -> true;
 keep_sel_item({_,less,_}) -> true;
@@ -432,9 +429,7 @@ change_texture_id(NewId, MatName, GeomSt0=#st{mat=Materials}) ->
 %%%% Menus.
 
 command_menu(body, X, Y) ->
-    Menu = [{basic,{?__(1,"Chart operations"),ignore}},
-	    {basic,separator},
-	    {?__(2,"Move"), move, ?__(3,"Move selected charts")},
+    Menu = [{?__(2,"Move"), move, ?__(3,"Move selected charts")},
 	    {?__(4,"Scale"), {scale, scale_directions(false) ++ 
 			      [separator] ++ stretch_directions() ++
 			      [separator, 
@@ -479,9 +474,7 @@ command_menu(body, X, Y) ->
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(face, X, Y) ->
     Scale = scale_directions(true),
-    Menu = [{basic,{?__(46,"Face operations"),ignore}},
-	    {basic,separator},
-	    {?__(47,"Move"),move,?__(48,"Move selected faces"),[magnet]},
+    Menu = [{?__(47,"Move"),move,?__(48,"Move selected faces"),[magnet]},
 	    {?__(49,"Scale"),{scale,Scale},?__(50,"Scale selected faces"), [magnet]},
 	    {?__(51,"Rotate"),rotate,?__(52,"Rotate selected faces"), [magnet]},
 	    separator,
@@ -495,9 +488,7 @@ command_menu(edge, X, Y) ->
 	[{?__(53,"Free"),free,?__(54,"Rotate selection freely"), [magnet]},
 	 {?__(55,"Chart to X"), align_x, ?__(56,"Rotate chart to align selected edge to X-axis")},
 	 {?__(57,"Chart to Y"), align_y, ?__(58,"Rotate chart to align selected edge to Y-axis")}],
-    Menu = [{basic,{?__(59,"Edge operations"),ignore}},
-	    {basic,separator},
-	    {?__(60,"Move"),move,?__(61,"Move selected edges"),[magnet]},
+    Menu = [{?__(60,"Move"),move,?__(61,"Move selected edges"),[magnet]},
 	    {?__(62,"Scale"),{scale,Scale},?__(63,"Scale selected edges"), [magnet]},
 	    {?__(64,"Rotate"),{rotate,Align},?__(65,"Rotate commands")},
 	    {?__(641,"Slide"),slide,?__(642,"Slide along neighbor edges")},
@@ -520,9 +511,7 @@ command_menu(vertex, X, Y) ->
 	 {?__(74,"Chart to Y"), align_y, 
 	  ?__(75,"Rotate chart to align (imaginary) edge joining selected verts to Y-axis")}],
 
-    Menu = [{basic,{?__(76,"Vertex operations"),ignore}},
-	    {basic,separator},
-	    {?__(77,"Move"),move,?__(78,"Move selected vertices"),[magnet]},
+    Menu = [{?__(77,"Move"),move,?__(78,"Move selected vertices"),[magnet]},
 	    {?__(79,"Scale"),{scale,Scale},?__(80,"Scale selected vertices"), [magnet]},
 	    {?__(81,"Rotate"),{rotate,Align},?__(82,"Rotation commands")},
 	    separator,
@@ -703,9 +692,9 @@ handle_event_3({action,{auv,{draw_options,Opt}}}, #st{bb=Uvs}=St) ->
 	    get_event(St#st{bb=Uvs#uvstate{st=GeomSt,matname=MatName}})
     end;
 %% Others
-handle_event_3({vec_command,Command,_St}, _) when is_function(Command) ->
+handle_event_3({vec_command,Command,_St}, _) when is_function(Command, 0) ->
     %% Use to execute command with vector arguments (see wings_vec.erl).
-    catch Command();
+    Command();
 handle_event_3(close, _St) ->
     cleanup_before_exit(),
     delete;
@@ -759,6 +748,28 @@ handle_event_3({action,Ev}, St) ->
 	    St1 = fake_selection(St),
 	    wings_view:command(aim, St1),
 	    get_event(St);
+	{view,highlight_aim} ->
+        #st{sel=Sel} = St,
+        case  Sel =:= [] of
+          true ->
+		    St1 = fake_selection(St),
+            wings_view:command(aim, St1),
+            get_event(St);
+          false ->
+            HL0 = wings_pref:get_value(highlight_aim_at_unselected),
+            HL1 = wings_pref:get_value(highlight_aim_at_selected),
+            {_,X,Y} = wings_wm:local_mouse_state(),
+            {{_,Cmd},St1} = case wings_pick:do_pick(X, Y, St) of
+                  {add,_,St2} when HL0 =:= true ->
+                      {{view,highlight_aim},{add,St,St2}};
+                  {delete,_,St2} when HL1 =:= true ->
+                      {{view,highlight_aim},{delete,St,St2}};
+                  _Other -> 
+                      {{view,aim}, St}
+            end,
+            wings_view:command(Cmd,St1),
+            get_event(St)
+        end;
 	{view,Cmd} when Cmd == frame ->
 	    wings_view:command(Cmd,St),
 	    get_event(St);
@@ -1839,7 +1850,9 @@ remap(proj_lsqcm, _, Sel, We0, St = #st{selmode=face}) ->
 		      {V,{S,T}}
 		  end || V <- SelVs],
 	remap(lsqcm, Pinned, Sel, We0, St)
-    catch throw:What ->
+    catch throw:{_,What} ->
+	    wpa:error(What);
+	      throw:What ->
 	    wpa:error(What)
     end;
 remap(Type, Pinned, _, We0, St) ->
