@@ -3,24 +3,26 @@
 %%
 %%     Plug-in for bending vertices
 %%
-%%  Copyright (c) 2005 Dave Rodgers
+%%  Copyright (c) 2005-2009 Dave Rodgers
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_bend.erl 243 2008-02-18 05:51:34Z optigon $
+%%     $Id$
 %%
 
 
 -module(wpc_bend).
 
 -export([init/0,menu/2,command/2]).
--import(lists, [foldl/3,splitwith/2]).
+-import(lists, [foldl/3]).
 -include("wings.hrl").
 -include("e3d.hrl").
 
 -define(HUGE, 1.0E307).
--define(DEBUG_BEND, false).
+
+%% Uncomment the following line to turn on debugging.
+%% -define(DEBUG_BEND, 1).
 
 -record(bend_data, {dragMode,          % fixed_length or fixed_radius
                     rodCenter,
@@ -42,11 +44,9 @@ init() ->
     true.
 
 menu({vertex}, Menu) ->
-    Menu ++ [{advanced,separator},
-	     {advanced, {?__(1,"Bend"),
-			 {bend,fun adv_submenu_noclamp/2}}},
-	     {advanced, {?__(2,"Bend Clamped"),
-			 {bend,fun adv_submenu_clamped/2}}}];
+    Menu ++ [separator,
+	     {?__(1,"Bend"),{bend,fun adv_submenu_noclamp/2}},
+	     {?__(2,"Bend Clamped"),{bend,fun adv_submenu_clamped/2}}];
 menu(_,Menu) -> Menu.
 
 command({vertex,{bend,Type}}, St) ->
@@ -298,10 +298,8 @@ bend_vertex(Pos, Angle, #bend_data{dragMode = DragMode,
                                    pivotNormal = PN,
                                    posHeightClamp = PC,
                                    negHeightClamp = NC}=BD) ->
-  case ?DEBUG_BEND of
-    true  -> io:format(?__(1,"Angle")++" = ~p: ", [Angle]), print_bend_data(BD);
-    false -> []
-  end,
+
+  maybe_print_bend_data(Angle, BD),
 
   Radians = Angle * (math:pi()/180.0),
 
@@ -356,13 +354,12 @@ bend_vertex(Pos, Angle, #bend_data{dragMode = DragMode,
 %%  Utilities
 %%
 
-vectorsToArray(Vectors) ->
-  vectorsToArray(Vectors, []).
-vectorsToArray([{X,Y,Z}|T], Acc) ->
-  vectorsToArray(T, Acc ++ [X,Y,Z]);
-vectorsToArray([], Acc) ->
-  Acc.
-
+-ifndef(DEBUG_BEND).
+maybe_print_bend_data(_, _) -> ok.
+-else.
+maybe_print_bend_data(Angle, BD) ->
+    io:format("Angle = ~p: ", [Angle]),
+    print_bend_data(BD).
 
 print_bend_data(#bend_data{dragMode = DragMode,
                            rodCenter = RC,
@@ -385,3 +382,11 @@ print_bend_data(#bend_data{dragMode = DragMode,
   io:format("  rodLength = ~p\n", [RL]),
   io:format("  posHeightClamp = ~p\n", [PC]),
   io:format("  negHeightClamp = ~p\n", [NC]).
+
+vectorsToArray(Vectors) ->
+  vectorsToArray(Vectors, []).
+vectorsToArray([{X,Y,Z}|T], Acc) ->
+  vectorsToArray(T, Acc ++ [X,Y,Z]);
+vectorsToArray([], Acc) ->
+  Acc.
+-endif.
