@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_doosabin.erl 199 2008-01-03 10:13:25Z bjorng $
+%%     $Id$
 %%
 
 -module(wpc_doosabin).
@@ -37,9 +37,10 @@ menu_entry(T) ->
       ?__(2,"Makes a Doo-Sabin subdivision according to WasaMonkey (WARNING: messes up all UV coordinates)")}|T].
 
 command({body,doo_sabin}, St0) ->
-    %% Do For each selected object 
-    wpa:sel_fold(fun(_,We,St) -> doo_sabin(We,St) end, 
-		 St0#st{sel=[],selmode=edge}, St0);
+    %% Do for each selected object
+    St = wpa:sel_fold(fun(_,We,St) -> doo_sabin(We,St) end,
+		      St0#st{sel=[],selmode=edge}, St0),
+    wpa:sel_set(St#st.sel, St);
 command(_Cmd, _) -> next.
 
 doo_sabin(We0 = #we{es=Etab0, id=Id}, St0 = #st{shapes=Sh0,sel=OrigSel}) ->
