@@ -8,12 +8,12 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wp9_dialogs.erl 177 2007-12-21 18:56:58Z bjorng $
+%%     $Id$
 %%
 
 -module(wp9_dialogs).
 -export([init/1]).
--import(lists, [reverse/1,reverse/2,sort/1]).
+-import(lists, [sort/1]).
 
 -include("wings_intl.hrl").
 
