@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_test_ask.erl 211 2008-01-06 08:06:06Z bjorng $
+%%     $Id$
 %%
 
 -module(wpc_test_ask).
@@ -19,7 +19,7 @@
 -export([load/1]).
 -export([init/0,menu/2,dialog/2,command/2]).
 
--import(lists, [reverse/1,reverse/2]).
+-import(lists, [reverse/2]).
 
 enable() -> wpa:pref_set(?MODULE, enabled, true).
 
