@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings.erl 294 2008-03-22 02:12:54Z optigon $
+%%     $Id$
 %%
 
 -module(wings).
@@ -16,6 +16,8 @@
 -export([redraw/1,redraw/2,init_opengl/1,command/2]).
 -export([mode_restriction/1,clear_mode_restriction/0,get_mode_restriction/0]).
 -export([ask/3]).
+-export([save_windows/0]).
+-export([handle_drop/3]).
 -export([init_menubar/0]).
 
 -export([register_postdraw_hook/3,unregister_postdraw_hook/2]).
@@ -24,8 +26,7 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 
--import(lists, [foreach/2,map/2,filter/2,foldl/3,sort/1,
-        keymember/3,reverse/1]).
+-import(lists, [foldl/3,sort/1]).
 
 start() ->
     do_spawn(none).
@@ -232,6 +233,9 @@ unregister_postdraw_hook(Window, Id) ->
         ok
     end.
 
+save_state(St, St) ->
+    main_loop(St);
+
 save_state(St0, St1) ->
     St2 = wings_undo:save(St0, St1),
     St = case St2 of
@@ -367,7 +371,7 @@ handle_event_3(got_focus, _) ->
     Msg3 = wings_msg:button_format([], [], ?__(2,"Show menu")),
     Message = wings_msg:join([Msg1,Msg2,Msg3]),
     wings_wm:message(Message),
-    wings_wm:dirty();
+    keep;
 handle_event_3(lost_focus, _) -> keep;
 handle_event_3({note,menu_aborted}, St) ->
     main_loop(clear_temp_sel(St));
@@ -380,6 +384,10 @@ handle_event_3(language_changed, _) ->
     wings_wm:toplevel_title(This, geom_title(This)),
     wings_wm:menubar(This, get(wings_menu_template)),
     keep;
+handle_event_3({external,no_more_basic_menus}, _St) ->
+    wings_help:no_more_basic_menus();
+handle_event_3({external,launch_tweak}, St) ->
+    wpc_tweak:command({tools,{tweak,false}},St);
 handle_event_3({external,Op}, St) ->
     wpa:handle_external(Op,St),
     keep;
@@ -395,6 +403,7 @@ do_hotkey(Ev, #st{sel=[]}=St0) ->
         {add,_,St} ->
             case wings_hotkey:event(Ev, St) of
             next -> next;
+            {view,highlight_aim} -> {{view,aim},set_temp_sel(St0, St)};
             Cmd ->
                 case highlight_sel_style(Cmd) of
                 none -> {Cmd,St0};
@@ -405,11 +414,32 @@ do_hotkey(Ev, #st{sel=[]}=St0) ->
         _Other -> do_hotkey_1(Ev, St0)
         end
     end;
-do_hotkey(Ev, St) -> do_hotkey_1(Ev, St).
+
+%%%% Hack to get a basic form of Highlight Aim (see wings_view:highlight_aim/2)
+do_hotkey(Ev, St0) ->
+    case wings_pref:get_value(use_temp_sel) of
+      false ->
+        do_hotkey_1(Ev, St0);
+      true ->
+	    Hs = wings_pref:get_value(hilite_select),
+        case wings_hotkey:event(Ev, St0) of
+          next -> next;
+          {view,highlight_aim} -> highlight_aim_setup(St0);
+          {select,{edge_loop,edge_loop}}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,{edge_loop,edge_ring}}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,{oriented_faces,_}}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,{similar_material,_}}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,{similar_area,_}}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,similar}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          {select,all}=Cmd when Hs -> hotkey_select_setup(Cmd,St0);
+          Cmd -> {Cmd,St0}
+        end
+    end.
 
 do_hotkey_1(Ev, St) ->
     case wings_hotkey:event(Ev, St) of
     next -> next;
+    {view,highlight_aim} -> {{view,aim}, St};
     Cmd -> {Cmd,St}
     end.
 
@@ -420,14 +450,24 @@ highlight_sel_style({body,_}) -> temporary;
 highlight_sel_style({edit,repeat}) -> temporary;
 highlight_sel_style({edit,repeat_args}) -> temporary;
 highlight_sel_style({edit,repeat_drag}) -> temporary;
+highlight_sel_style({select,deselect}) -> none;
 highlight_sel_style({select,vertex}) -> none;
 highlight_sel_style({select,edge}) -> none;
 highlight_sel_style({select,face}) -> none;
 highlight_sel_style({select,body}) -> none;
 highlight_sel_style({select,{adjacent,_}}) -> none;
 highlight_sel_style({select,_}) -> permanent;
+highlight_sel_style({tools,{virtual_mirror,_}}) -> temporary;
+highlight_sel_style({tools,{align,_}}) -> temporary;
+highlight_sel_style({tools,{center,_}}) -> temporary;
+highlight_sel_style({tools,put_on_ground}) -> temporary;
+highlight_sel_style({tools,save_bb}) -> temporary;
+highlight_sel_style({tools,{scale_to_bb,_}}) -> temporary;
+highlight_sel_style({tools,{scale_to_bb_prop,_}}) -> temporary;
+highlight_sel_style({tools,{move_to_bb,_}}) -> temporary;
 highlight_sel_style({view,align_to_selection}) -> temporary;
 highlight_sel_style({view,aim}) -> temporary;
+highlight_sel_style({view,highlight_aim}) -> temporary;
 highlight_sel_style({view,frame}) -> temporary;
 
 highlight_sel_style(_) -> none.
@@ -483,15 +523,21 @@ remember_command(_Cmd, St) -> St.
 %% Test if the saved command can be safely repeated, and
 %% rewrite it with the current selection mode if needed.
 repeatable(Mode, Cmd) ->
+%   io:format("Cmd ~p\n",[Cmd]),
     case Cmd of
     {Mode,_} -> Cmd;			%Same mode is always OK.
 
     %% Commands safe in all modes.
     {_,{move,normal}} when Mode == body -> no;
+    {_,{_,region}} when Mode =/= face -> no;
     {_,{move,_}=C} -> {Mode,C};
-    {_,{rotate,normal}} when Mode == body -> no;
+    {_,{rotate,{_,{_,[normal],_}}}}when Mode == vertex; Mode == body -> no;
+    {_,{rotate,{_,{_,[_,normal],_}}}}when Mode == vertex; Mode == body -> no;
     {_,{rotate,_}=C} -> {Mode,C};
     {_,{scale,_}=C} -> {Mode,C};
+    {_,{move_planar,_}=C} -> {Mode,C};
+    {_,{absolute,_}=C} -> {Mode,C};
+    {_,{arc_intersect,_}=C} -> {Mode,C};
 
     %% Some special cases.
     {_,tighten=C} when Mode == vertex; Mode == body -> {Mode,C};
@@ -653,6 +699,9 @@ command({tools, screenshot}, St) ->
 command({tools, area_volume_info}, St) ->
     area_volume_info(St),
     St;
+command({tools, scene_size_info}, St) ->
+    scene_size_info(St),
+    St;
 command({tools, put_on_ground}, St) ->
     {save_state,wings_align:put_on_ground(St)};
 command({tools, unitize}, St) ->
@@ -754,6 +803,8 @@ tools_menu(_) ->
      separator,
      {?__(26,"Scene Info: Area & Volume"), area_volume_info,
       ?__(27,"Calculate area and volume for each object in the scene")},
+     {?__(32,"Memory Usage"), scene_size_info,
+      ?__(33,"Calculate the memory usage for the current scene from the saved state")},
      {?__(28,"Put on Ground"), put_on_ground,
       ?__(29,"Put selected objects on the ground plane")},
      {?__(30,"Unitize"), unitize,
@@ -815,21 +866,18 @@ purge_undo(St) ->
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
 info(#st{sel=[]}) ->
-    [];
-%% XXX I have turned off shaders for now.
-%%  -- Bjorn G
-%% 
-%%     Progs = get(light_shaders),
-%%     NumLights = wings_pref:get_value(number_of_lights),
-%%     NumShaders = wings_pref:get_value(number_of_shaders),
-%%     UseProg = (Progs /= undefined) and (NumLights == 2),
-%%     case UseProg of
-%% 	true ->
-%% 	    {_Prog,Name} = element(NumShaders, Progs),
-%% 	    io_lib:format("Shader ~p of ~p: ~s ",[NumShaders,tuple_size(Progs),Name]);
-%% 	false ->
-%% 	    []
-%%     end;
+    [],
+    Progs = get(light_shaders),
+    NumLights = wings_pref:get_value(number_of_lights),
+    NumShaders = wings_pref:get_value(number_of_shaders),
+    UseProg = (Progs /= undefined) and (NumLights == 2),
+    case UseProg of
+     true ->
+	 {_Prog,Name} = element(NumShaders, Progs),
+	 io_lib:format("Shader ~p of ~p: ~s ",[NumShaders,tuple_size(Progs),Name]);
+     false ->
+	 []
+    end;
 info(St) ->
     case wings_wm:get_prop(show_info_text) of
     false -> [];
@@ -1495,10 +1543,11 @@ get_mode_restriction() ->
 
 area_volume_info(St) ->
     #st{shapes=Shapes} = St,
-    Header = io_lib:fwrite("~s: '~s' [~s] (~s)\n", ["#", "Object Name", "Area", "Volume"]),
+    Header = io_lib:fwrite("~s: '~s' [~s] (~s)\n", ["#", ?__(1,"Object Name"),
+                           ?__(2,"Area"),?__(3, "Volume")]),
     Info = [get_object_info(Id, Shapes) || Id <- gb_trees:keys(Shapes)],
     Msg = lists:flatten(Header ++ Info),
-    wings_help:help_window("Scene Info: Area & Volume", [Msg]).
+    wings_help:help_window(?__(5,"Scene Info: Area & Volume"), [Msg]).
 
 get_object_info(Id, Shapes) ->
     We0 = gb_trees:get(Id, Shapes),
@@ -1519,3 +1568,32 @@ area_volume(Face, We) ->
     Volume = e3d_vec:dot(V1, Bc)/6.0,
     {Area, Volume}.
 
+scene_size_info(St) ->
+    Bytes = byte_size(term_to_binary(St)),
+    MB = Bytes/1048576,
+	{Unit,Amount} = best_unit(Bytes,MB),
+    Usage = io_lib:format(?__(1,"Your scene is using approximately ~s ~s [~p bytes]"),
+            [wings_util:nice_float(Amount), Unit, Bytes]),
+    wings_help:help_window(?__(2,"Memory Usage"), [Usage]).
+
+best_unit(Bytes,MB) when  MB < 1.0 ->
+    {"KB",Bytes/1024};
+best_unit(_, MB) ->
+    {"MB",MB}.
+
+highlight_aim_setup(St0) ->
+    HL0 = wings_pref:get_value(highlight_aim_at_unselected),
+    HL1 = wings_pref:get_value(highlight_aim_at_selected),
+    {_,X,Y} = wings_wm:local_mouse_state(),
+    case wings_pick:do_pick(X, Y, St0) of
+      {add,_,St} when HL0 =:= true -> {{view,highlight_aim},{add,St0,St}};
+      {delete,_,St} when HL1 =:= true -> {{view,highlight_aim},{delete,St0,St}};
+      _Other           -> {{view,aim}, St0}
+    end.
+
+hotkey_select_setup(Cmd,St0) ->
+    {_,X,Y} = wings_wm:local_mouse_state(),
+    case wings_pick:do_pick(X, Y, St0) of
+      {add,_,St} -> {Cmd,St};
+      _Other     -> {Cmd,St0}
+    end.
