@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_align.erl 179 2008-01-01 11:55:41Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_align).
@@ -17,7 +17,7 @@
 	 unitize/1]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldr/3,foldl/3,reverse/1]).
+-import(lists, [map/2,foldl/3,reverse/1]).
 
 align(_Axis, #st{sel=[]}=St) -> St;
 align(Axis, St) ->
@@ -163,6 +163,7 @@ put_obj_on_ground(We) ->
 unitize(St) ->
     wings_sel:map(fun(_, We) -> unitize_obj(We) end, St).
 
+unitize_obj(We) when ?IS_LIGHT(We) -> We;
 unitize_obj(We) ->
     [Min,Max] = wings_vertex:bounding_box(We),
     Size = e3d_vec:sub(Max, Min),
