@@ -3,12 +3,12 @@
 %%
 %%     This module handles camera moves (rotation, zooming, and panning).
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_camera.erl 313 2008-04-02 20:41:39Z optigon $
+%%     $Id$
 %%
 
 -module(wings_camera).
@@ -16,14 +16,12 @@
 
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
-
--import(lists, [foreach/2,map/2,foldl/3,sort/1,reverse/1,append/1]).
+-import(erlang, [max/2]).
 
 -define(ZOOM_FACTOR, 20).
 -define(ZOOM_FACTOR_ALT, 1).
 
 -define(CAMDIV, 4).
--define(CAMMAX, 150).  %% Always larger than 300 on my pc
 
 -define(CSEP, 160).				%Short space.
 -define(NEARZERO, 1.0e-6).
@@ -36,9 +34,11 @@
 -record(state, {st, func}).
 
 init() ->
-    wings_pref:set_default(camera_mode, mirai),
+    wings_pref:set_default(camera_mode, wings_cam),
     wings_pref:set_default(num_buttons, 3),
+    wings_pref:set_default(cam_rotation_speed, 25),
     wings_pref:set_default(pan_speed, 25),
+    wings_pref:set_default(pan_using_arrow_keys, true),
     wings_pref:set_default(pan_speed_arrow_keys, 50),
     wings_pref:set_default(inverted_wheel_zoom, false),
     wings_pref:set_default(wheel_adds, false),
@@ -56,13 +56,16 @@ prefs() ->
     ZoomFlag0 = wings_pref:get_value(wheel_zooms, true),
     ZoomFactor0 = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR),
     ZoomFactorAlt = wings_pref:get_value(wheel_zoom_factor_alt, ?ZOOM_FACTOR_ALT),
-    PanSpeed0 = wings_pref:get_value(pan_speed),
-    ArrowPanSpeed = wings_pref:get_value(pan_speed_arrow_keys),
-    InvertZW = wings_pref:get_value(inverted_wheel_zoom),
+    CamRotSpeed = wings_pref:get_value(cam_rotation_speed, 25),
+    PanSpeed0 = wings_pref:get_value(pan_speed, 25),
+    ArrowPanSpeed = wings_pref:get_value(pan_speed_arrow_keys, 50),
+    InvertZW = wings_pref:get_value(inverted_wheel_zoom, false),
     WheelAdds = wings_pref:get_value(wheel_adds,false),
     WhScrollInfo = wings_pref:get_value(wh_scroll_info,true),
-    WhPanSpd = wings_pref:get_value(wh_pan_spd),
-    WhRotate = wings_pref:get_value(wh_rot_spd),
+    ArrowKeysPan = wings_pref:get_value(pan_using_arrow_keys,true),
+    WhPanSpd = wings_pref:get_value(wh_pan_spd, 50),
+    WhRotate = wings_pref:get_value(wh_rot_spd, 7.5),
+
     Hook = fun (is_disabled, {_Var,_I,Sto}) ->
             not gb_trees:get(wheel_zooms, Sto);
              (_, _) -> void
@@ -71,44 +74,55 @@ prefs() ->
              not ((gb_trees:get(wheel_adds, Sto)) andalso (gb_trees:get(wheel_zooms, Sto)));
              (_, _) -> void
             end,
-    {vframe,
+    ArrowKeyHook = fun (is_disabled, {_Var,_I,Sto}) ->
+            not gb_trees:get(pan_using_arrow_keys, Sto);
+             (_, _) -> void
+           end,
+    {hframe,
+    [{vframe,
      [{vframe,[mouse_buttons()],[{title,?__(1,"Mouse Buttons")}]},
       {vframe,[camera_modes()],[{title,?__(2,"Camera Mode")}]},
+      {vframe,
+       [{hframe,[{slider,{text,CamRotSpeed,[{key,cam_rotation_speed},{range,{1,100}}]}}]}],
+       [{title,?__(19,"Rotation Speed")}]},
       {vframe,
        [{hframe,[{slider,{text,PanSpeed0,[{key,pan_speed},{range,{1,100}}]}}]}],
        [{title,?__(3,"Pan Speed")}]},
       {vframe,
-       [{hframe,[{slider,{text,ArrowPanSpeed,[{key,pan_speed_arrow_keys},{range,{1,100}}]}}]}],
-       [{title,?__(16,"Arrow Key Pan Speed")}]},
-      {vframe,
+       [{?__(18,"Arrow Keys Pan"),ArrowKeysPan,[{key,pan_using_arrow_keys}]},
+       {hframe,[{slider,{text,ArrowPanSpeed,[{key,pan_speed_arrow_keys},{range,{1,100}},
+        {hook,ArrowKeyHook}]}}]}],
+       [{title,?__(16,"Arrow Key Pan Speed")}]}]},
+    {vframe,
+      [{vframe,
        [{?__(4,"Wheel Zooms"),ZoomFlag0,[{key,wheel_zooms}]},
-	{vradio,[{?__(5,"Forwards Zooms In"),false},
-	         {?__(6,"Forwards Zooms Out"),true}],
-	 InvertZW,
-	 [{key,inverted_wheel_zoom},{hook,Hook}]},
-	{hframe,
-	 [{label,?__(7,"Zoom Factor"),[{hook,Hook}]},
-	  {text,ZoomFactor0,
-	   [{key,wheel_zoom_factor},
+    {vradio,[{?__(5,"Forwards Zooms In"),false},
+             {?__(6,"Forwards Zooms Out"),true}],
+     InvertZW,
+     [{key,inverted_wheel_zoom},{hook,Hook}]},
+    {hframe,
+     [{label,?__(7,"Zoom Factor"),[{hook,Hook}]},
+      {text,ZoomFactor0,
+       [{key,wheel_zoom_factor},
      {range,{1,50}},{hook,Hook}]},
-	  {label,"%",[{hook,Hook}]}]},
-	 {hframe,
-	  [{label,?__(10,"Alternate Zoom Factor(Alt+Scroll)"),[{hook,Hook}]},
-	   {text,ZoomFactorAlt,
-	    [{key,wheel_zoom_factor_alt},
+      {label,"%",[{hook,Hook}]}]},
+     {hframe,
+      [{label,?__(10,"Alternate Zoom Factor(Alt+Scroll)"),[{hook,Hook}]},
+       {text,ZoomFactorAlt,
+        [{key,wheel_zoom_factor_alt},
       {range,{1,50}},{hook,Hook}]},
      {label,"%",[{hook,Hook}]}]} ],[{title,?__(9,"Scroll Wheel")}]},
     {vframe,
-	  [{?__(11,"Wheel Pans & Rotates"),WheelAdds,[{key,wheel_adds},{hook,Hook}]},
-	   {?__(17,"Show Info Line Help String"),WhScrollInfo,[{key,wh_scroll_info},{hook,WHook}]}, 
-	  {vframe,
-	   [{hframe,
-	    [{slider,{text,WhPanSpd,[{key, wh_pan_spd},{range,{1,100}}]}}],
-	  [{title,?__(12,"Pan Speed")},{hook,WHook}]},
-	  {hframe,
-	   [{slider,{text,WhRotate,[{key, wh_rot_spd},{range,{?NEARZERO,180.0}}]}}],
-	   [{title,?__(13,"Rotation Step in Degrees")},{hook,WHook}]}]}],
-    [{title,?__(15,"Unidirectional Camera")}]}]}.
+      [{?__(11,"Wheel Pans & Rotates"),WheelAdds,[{key,wheel_adds},{hook,Hook}]},
+       {?__(17,"Show Info Line Help String"),WhScrollInfo,[{key,wh_scroll_info},{hook,WHook}]}, 
+      {vframe,
+       [{hframe,
+        [{slider,{text,WhPanSpd,[{key, wh_pan_spd},{range,{1,100}}]}}],
+      [{title,?__(12,"Pan Speed")},{hook,WHook}]},
+      {hframe,
+       [{slider,{text,WhRotate,[{key, wh_rot_spd},{range,{?NEARZERO,180.0}}]}}],
+       [{title,?__(13,"Rotation Step in Degrees")},{hook,WHook}]}]}],
+    [{title,?__(15,"Unidirectional Camera")}]}]}]}.
 
 mouse_buttons() ->
     {menu,[{desc(1),1,[{info,info(1)}]},
@@ -184,55 +198,20 @@ scroll_help() ->
 %% Event handler.
 event(Ev, St=#st{}) -> 
     event(Ev,St,none).
-%% Rotation event
-event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?SHIFT_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whrotate(0.5,0.0);
-event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?SHIFT_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whrotate(-0.5,0.0);
-event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?SHIFT_BITS =/= 0 ->
-    whrotate(0.0,0.5);
-event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?SHIFT_BITS =/= 0 ->
-    whrotate(0.0,-0.5);
-%% Pan event
-event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?CTRL_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whpan(0.5,0.0);
-event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?CTRL_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whpan(-0.5,0.0);
-event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?CTRL_BITS =/= 0 ->
-    whpan(0.0,0.5);
-event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?CTRL_BITS =/= 0 ->
-    whpan(0.0,-0.5);
-%% Zoom event
-event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?ALT_BITS =/= 0 ->
-    zoom_step_alt(-1);
-event(#mousebutton{button=4,state=?SDL_RELEASED}, _, _Redraw) ->
-    zoom_step(-1);
-event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _, _Redraw)
-  when Mod band ?ALT_BITS =/= 0 ->
-    zoom_step_alt(1);
-event(#mousebutton{button=5,state=?SDL_RELEASED}, _, _Redraw) ->
-    zoom_step(1);
-event(#mousebutton{button=B}, _, _Redraw) when B==4; B==5 ->
-    keep;
+%% Scroll wheel camera events
+event(#mousebutton{button=B}=Ev, _St, _Redraw) when B==4; B==5 ->
+    generic_event(Ev,_St,_Redraw);
+% Camera mode specific events
 event(Ev, St, Redraw) ->
     case wings_pref:get_value(camera_mode) of
+	wings_cam -> wings_cam(Ev, #state{st=St, func=Redraw});
 	blender -> blender(Ev, #state{st=St, func=Redraw});
 	nendo -> nendo(Ev, #state{st=St, func=Redraw});
 	mirai -> mirai(Ev, #state{st=St, func=Redraw});
 	tds -> tds(Ev, #state{st=St, func=Redraw});
 	maya -> maya(Ev, #state{st=St, func=Redraw});
 	mb -> mb(Ev, #state{st=St, func=Redraw});
-	sketchup -> sketchup(Ev, #state{st=St, func=Redraw});
-	wings_cam -> wings_cam(Ev, #state{st=St, func=Redraw})
+	sketchup -> sketchup(Ev, #state{st=St, func=Redraw})
     end.
 
 %%%
@@ -478,7 +457,7 @@ tds_help() ->
 %%%
 
 maya(#mousebutton{x=X0,y=Y0,mod=Mod,state=?SDL_PRESSED}, Redraw)
-  when Mod band ?ALT_BITS =/= 0 ->
+  when Mod band ?ALT_BITS =/= 0, Mod band ?CTRL_BITS =:= 0, Mod band ?SHIFT_BITS =:= 0 ->
     {X,Y} = wings_wm:local2global(X0, Y0),
     sdl_events:eventState(?SDL_KEYUP, ?SDL_ENABLE),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
@@ -698,16 +677,16 @@ generic_event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redr
 
 generic_event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redraw)
   when Mod band ?CTRL_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whpan(0.5,0.0);
+    whpan(0.05,0.0);
 generic_event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redraw)
   when Mod band ?CTRL_BITS =/= 0 andalso Mod band ?ALT_BITS =/= 0 ->
-    whpan(-0.5,0.0);
+    whpan(-0.05,0.0);
 generic_event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redraw)
   when Mod band ?CTRL_BITS =/= 0 ->
-    whpan(0.0,0.5);
+    whpan(0.0,0.05);
 generic_event(#mousebutton{button=5,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redraw)
   when Mod band ?CTRL_BITS =/= 0 ->
-    whpan(0.0,-0.5);
+    whpan(0.0,-0.05);
 
 generic_event(#mousebutton{button=4,mod=Mod,state=?SDL_RELEASED}, _Camera, _Redraw)
   when Mod band ?ALT_BITS =/= 0 ->
@@ -722,13 +701,14 @@ generic_event(#mousebutton{button=5,state=?SDL_RELEASED}, _Camera, _Redraw) ->
 generic_event(_, _, _) -> keep.
 
 rotate(Dx, Dy) ->
+    Speed = wings_pref:get_value(cam_rotation_speed,25)/25,
     case allow_rotation() of
 	false -> ok;
 	true ->
 	    View0= wings_view:current(),
 	    #view{azimuth=Az0,elevation=El0} = View0,
-	    Az = Az0 + Dx,
-	    El = El0 + Dy,
+	    Az = Az0 + Dx * Speed,
+	    El = El0 + Dy * Speed,
 	    View = View0#view{azimuth=Az,elevation=El,along_axis=none},
 	    wings_view:set_current(View)
     end.
@@ -743,7 +723,7 @@ whrotate(Dx, Dy) ->
              wings_wm:dirty(),
              View0= wings_view:current(),
              #view{azimuth=Az0,elevation=El0} = View0,
-             S = 2.0*wings_pref:get_value(wh_rot_spd),
+             S = 2 * wings_pref:get_value(wh_rot_spd),
              Az = Az0 + Dx*S,
              El = El0 + Dy*S,
              View = View0#view{azimuth=Az,elevation=El,along_axis=none},
@@ -756,34 +736,30 @@ zoom_step_alt(Dir) ->
     case wings_pref:get_value(wheel_zooms, true) of
 	false -> keep;
 	true ->
-	    wings_wm:dirty(),
-	    #view{distance=Dist} = View = wings_view:current(),
-	    ZoomPercent0 = wings_pref:get_value(wheel_zoom_factor_alt, ?ZOOM_FACTOR_ALT)/100,
-	    ZoomPercent = case wings_pref:get_value(inverted_wheel_zoom) of
-			      false -> ZoomPercent0;
-			      true -> -ZoomPercent0
-			  end,
-	    Delta = dist_factor(Dist)*Dir*ZoomPercent,
-	    wings_view:set_current(View#view{distance=Dist+Delta}),
-	    keep
+	    ZoomFactor = wings_pref:get_value(wheel_zoom_factor_alt, ?ZOOM_FACTOR_ALT),
+		wheel_zoom(ZoomFactor,Dir)
     end.
 
 zoom_step(Dir) ->
     case wings_pref:get_value(wheel_zooms, true) of
 	false -> keep;
 	true ->
-	    wings_wm:dirty(),
-	    #view{distance=Dist} = View = wings_view:current(),
-	    ZoomPercent0 = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR)/100,
-	    ZoomPercent = case wings_pref:get_value(inverted_wheel_zoom) of
-			      false -> ZoomPercent0;
-			      true -> -ZoomPercent0
-			  end,
-	    Delta = dist_factor(Dist)*Dir*ZoomPercent,
-	    wings_view:set_current(View#view{distance=Dist+Delta}),
-	    keep
+	    ZoomFactor = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR),
+		wheel_zoom(ZoomFactor,Dir)
     end.
 
+wheel_zoom(Factor,Dir) ->
+    wings_wm:dirty(),
+    #view{distance=Dist} = View = wings_view:current(),
+    ZoomPercent0 = Factor/100,
+    ZoomPercent = case wings_pref:get_value(inverted_wheel_zoom) of
+		      false -> ZoomPercent0;
+		      true -> -ZoomPercent0
+		  end,
+    Delta = dist_factor(Dist)*Dir*ZoomPercent,
+    wings_view:set_current(View#view{distance=Dist+Delta}),
+    keep.
+
 zoom(Delta0) ->
     #view{distance=Dist} = View = wings_view:current(),
     Delta = dist_factor(Dist)*Delta0/80,
@@ -798,16 +774,22 @@ pan(Dx0, Dy0) ->
     PanY = PanY0 - Dy,
     wings_view:set_current(View#view{pan_x=PanX,pan_y=PanY}).
 
-arrow_key_pan(?SDLK_LEFT) ->
-    arrow_key_pan(0.25, 0.0);
-arrow_key_pan(?SDLK_RIGHT) ->
-    arrow_key_pan(-0.25, 0.0);
-arrow_key_pan(?SDLK_UP) ->
-    arrow_key_pan(0.0, 0.25);
-arrow_key_pan(?SDLK_DOWN) ->
-    arrow_key_pan(0.0, -0.25);
+arrow_key_pan(Key) when Key=:=?SDLK_LEFT; Key=:=?SDLK_RIGHT; Key=:=?SDLK_UP; Key=:=?SDLK_DOWN ->
+    case wings_pref:get_value(pan_using_arrow_keys) of
+	  true -> arrow_key_pan_1(Key);
+	  _other -> next
+	end;
 arrow_key_pan(_) -> next.
 
+arrow_key_pan_1(?SDLK_LEFT) ->
+    arrow_key_pan(0.05, 0.0);
+arrow_key_pan_1(?SDLK_RIGHT) ->
+    arrow_key_pan(-0.05, 0.0);
+arrow_key_pan_1(?SDLK_UP) ->
+    arrow_key_pan(0.0, 0.05);
+arrow_key_pan_1(?SDLK_DOWN) ->
+    arrow_key_pan(0.0, -0.05).
+
 arrow_key_pan(Dx, Dy) ->
     key_pan(Dx, Dy),
     wings_wm:dirty(),
@@ -841,7 +823,7 @@ whpan(Dx0, Dy0) ->
        end
     end.
 dist_factor(Dist) ->
-    wings_util:max(abs(Dist), 0.2).
+    max(abs(Dist), 0.2).
 
 stop_camera(#camera{ox=Ox,oy=Oy}) ->
     case wings_io:ungrab(Ox, Oy) of
