@@ -3,12 +3,12 @@
 %%
 %%     This module handles interactive commands.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_drag.erl 300 2008-03-24 20:03:25Z optigon $
+%%     $Id$
 %%
 
 -module(wings_drag).
@@ -18,8 +18,7 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
--import(lists, [foreach/2,map/2,foldl/3,sort/1,keysort/2,
-		reverse/1,reverse/2,member/2]).
+-import(lists, [foldl/3,sort/1,reverse/1,reverse/2,member/2]).
 
 %% Main drag record. Kept in state.
 -record(drag,
@@ -29,10 +28,12 @@
 	 ys=0,
 	 zs=0,                  %Z move in screen relative
 	 fp=0,                  %An optional forth drag parameter
+	 psum=[0,0,0,0],        % Whereas xs,ys,zs and fp are the displayed distances
+	                        % psum is the unconstrained mouse summary pre parameter 
 	 xt=0,					%Last warp length
 	 yt=0,
 	 mmb_count=0,
-	 fp_count=0,
+	 fp_count=0,            % Rmb depression timer (forth parameter)
 	 offset,				%Offset for each dimension.
 	 unit,					%Unit that drag is done in.
 	 unit_sc,				%Scales for each dimension.
@@ -118,8 +119,8 @@ custom_absolute_drag() ->
     Speed = wings_pref:get_value(drag_speed_abs),
     case wings_pref:get_value(drag_cam_dist_abs) of
       true ->
-        #view{distance=D} = wings_view:current(),
-        D/((11-Speed)*((11-Speed)*300));
+        #view{fov=Fov, distance=D} = wings_view:current(),
+        (D/((11-Speed)*((11-Speed)*300)))*Fov/60;
       false ->
         1/((11-Speed)*80)
     end.
@@ -129,7 +130,7 @@ custom_relative_drag() ->
     case wings_pref:get_value(drag_cam_dist_relative) of
       true ->
         #view{distance=D} = wings_view:current(),
-        D/((11-Speed)*((11-Speed)*900));
+        (D/((11-Speed)*((11-Speed)*900)));
       false ->
         1/((11-Speed)*100)
     end.
@@ -169,6 +170,12 @@ pad_offsets(Ds) ->
 	L -> Ds ++ lists:duplicate(4-L, 0.0)
     end.
 
+%% When the zoom changes during a camera event the units have to be rescaled.
+adjust_unit_scaling([D|Ds],[U|Us],[U0|Us0]) ->
+    [D*U0/U|adjust_unit_scaling(Ds,Us,Us0)];
+adjust_unit_scaling(Ds,[],[]) -> Ds;
+adjust_unit_scaling([],[],[]) -> [].
+
 %%
 %% Here we break apart the objects into two parts - static part
 %% (not moved during drag) and dynamic (part of objects actually
@@ -360,10 +367,10 @@ help_message(#drag{unit=Unit,mode_fun=ModeFun,mode_data=ModeData}) ->
     ZMsg = zmove_help(Unit),
     FpMsg = fpmove_help(Unit),
     Cancel = wings_msg:button_format([], [],wings_s:cancel()),
-    NumEntry = ?__(1,"Numeric entry"),
+    NumEntry = ?__(1,"Numeric Entry"),
     Tab = wings_util:key_format("[Tab]", NumEntry),
     Switch = switch(),
-    Constraint = ?__(2,"Switch constraint sets")++Switch,
+    Constraint = ?__(2,"Switch Constraint Set")++Switch,
     ShiftTab = wings_util:key_format("[Shift]+[Tab]", Constraint),
     Msg = wings_msg:join([Accept,ZMsg,FpMsg,Cancel,Tab,ShiftTab]),
     MsgRight = ModeFun(help, ModeData),
@@ -371,7 +378,7 @@ help_message(#drag{unit=Unit,mode_fun=ModeFun,mode_data=ModeData}) ->
 
 switch() ->
     case wings_pref:get_value(con_alternate) of
-      true -> ?__(1," (using alternate constraints)");
+      true -> ?__(1," (using Alternate Constraints)");
       false -> []
     end.
 	
@@ -381,7 +388,8 @@ zmove_help([_,_,falloff]) -> [];
 zmove_help([_,_,dz|_]) ->
     zmove_help_1(?__(1,"Drag to move along Z"));
 zmove_help([_,_,percent|_]) ->
-    zmove_help_1(?__(3,"Drag to adjust scale"));
+    zmove_help_1(?__(3,"Drag to adjust Scale"));
+zmove_help([_,_,skip|_]) -> [];
 zmove_help([_,_,_|_]) ->
     zmove_help_1(?__(2,"Drag to adjust third parameter")).
 
@@ -392,7 +400,13 @@ fpmove_help([_,_,_,falloff]) -> [];
 fpmove_help([_,_,_,_,falloff]) ->
     fpmove_help_1(?__(1,"Drag to adjust forth parameter"));
 fpmove_help([_,_,_,angle|_]) ->
-    fpmove_help_1(?__(2,"Drag to adjust rotation"));
+    fpmove_help_1(?__(2,"Drag to adjust Rotation"));
+fpmove_help([_,_,_,percent|_]) ->
+    fpmove_help_1(?__(3,"Drag to adjust Scale"));
+fpmove_help([_,_,skip,Type|_]) ->
+    Str = wings_util:stringify(Type),
+    Help = ?__(4,"Drag to adjust") ++ " " ++ Str,
+    fpmove_help_1(Help);
 fpmove_help([_,_,_,_|_]) ->
     fpmove_help_1(?__(1,"Drag to adjust forth parameter")).
 
@@ -419,6 +433,15 @@ get_drag_event(Drag) ->
 get_drag_event_1(Drag) ->
     {replace,fun(Ev) -> handle_drag_event(Ev, Drag) end}.
 
+%%%% When the Rmb is pressed we store the time in order to determine if the
+%%%% intention is to drag or cancel the command. Later, when the Rmb is released
+%%%% we take the time again. If the time interval is short then the event is
+%%%% handled as an intention to cancel the command.
+handle_drag_event(#mousebutton{button=3,state=?SDL_PRESSED}=Ev,
+		  #drag{fp_count=C}=Drag) when C == 0 ->
+	StartTimer = now(),
+    handle_drag_event(Ev, Drag#drag{fp_count=StartTimer});
+
 handle_drag_event(#keyboard{sym=9, mod=Mod},Drag)->
     case Mod band ?SHIFT_BITS =/= 0 of
       true -> 
@@ -429,19 +452,42 @@ handle_drag_event(#keyboard{sym=9, mod=Mod},Drag)->
       false -> numeric_input(Drag)
     end;
 handle_drag_event(#mousebutton{button=2,state=?SDL_RELEASED},
-		  #drag{mmb_count=C}=Drag) when C > 2 ->
-    get_drag_event_1(Drag#drag{mmb_count=0});
+		  #drag{mmb_count=C,fp_count=0}=Drag) when C > 2 ->
+    get_drag_event_1(Drag#drag{mmb_count=0,fp_count=0});
 handle_drag_event(#mousebutton{button=3,state=?SDL_RELEASED,mod=Mod}=Ev,
 		  #drag{mmb_count=C}=Drag) when C > 2 ->
     if
 	Mod band ?CTRL_BITS =/= 0 ->
-	    get_drag_event_1(Drag#drag{mmb_count=0});
+	    get_drag_event_1(Drag#drag{mmb_count=0,fp_count=0});
 	true ->
-	    handle_drag_event_0(Ev, Drag)
+	    handle_drag_event_0(Ev,Drag#drag{fp_count=0})
     end;
+
+%%%% When Rmb is released we subtract the StartTime (when the Rmb was pressed)
+%%%% from the Stop time (relased) and if the result is less than 16000 ms, then
+%%%% we cancel the drag.
 handle_drag_event(#mousebutton{button=3,state=?SDL_RELEASED},
-		  #drag{fp_count=C}=Drag) when C > 2 ->
-    get_drag_event_1(Drag#drag{fp_count=0});
+          #drag{fp_count=StartTime}=Drag) when StartTime=/=0->
+    Stop = now(),
+    Time = timer:now_diff(Stop, StartTime),
+    % io:format("Time ~p\n",[Time]),
+    case Time < 250000 of
+        false ->
+            get_drag_event_1(Drag#drag{fp_count=0});
+        true ->
+            wings_dl:map(fun invalidate_fun/2, []),
+            ungrab(Drag),
+            wings_wm:later(revert_state),
+            pop
+	end;
+handle_drag_event(#mousebutton{button=3,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Drag0) ->
+%%%% This function guards against reported crashes of the rmb being held, and
+%%%% clicking the lmb. I can't reproduce this crash, but I don;t doubt that it
+%%%% happens. The probable cause is likely to do with misinterpreted or conflated
+%%%% mouse button hits. ~Richard Jones
+    Ev = #mousemotion{x=X,y=Y,state=0,mod=Mod},
+    Drag = ?SLOW(motion(Ev, Drag0)),
+    quit_drag(Drag);
 
 handle_drag_event(Event, Drag = #drag{st=St}) ->
     case wings_camera:event(Event, St, fun() -> redraw(Drag) end) of
@@ -452,12 +498,13 @@ handle_drag_event(Event, Drag = #drag{st=St}) ->
 	    wings_dl:map(fun(#dlo{hilite=none}=D, _) -> D;
 			    (D, _) -> D#dlo{hilite=none}
 			 end, []),
-	    %% Recalc unit_scales since zoom can have changed.
-	    #drag{xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,unit=Unit,unit_sc=[US0|_]} = Drag,
-	    US = [US1|_] = unit_scales(Unit),
-	    Adjust = US0/US1,
-	    get_drag_event(Drag#drag{xs=Xs0*Adjust,ys=Ys0*Adjust,zs=Zs0*Adjust,
-		               fp=Fp0*Adjust,unit_sc=US});
+	    %% Recalc unit_scales since zoom can have changed.   UNITS ARE MIXED IN SOME CASES
+	    #drag{xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,psum=Psum0,unit=Unit,
+	          unit_sc=Us0} = Drag,
+	    Us = unit_scales(Unit),
+		Psum = adjust_unit_scaling(Psum0,Us,Us0),
+		[Xs,Ys,Zs,Fp] = adjust_unit_scaling([Xs0,Ys0,Zs0,Fp0],Us,Us0),
+	    get_drag_event(Drag#drag{xs=Xs,ys=Ys,zs=Zs,fp=Fp,psum=Psum,unit_sc=Us});
 	Other ->
 	    %% Clear any potential marker for an edge about to be
 	    %% cut (Cut RMB).
@@ -475,17 +522,23 @@ handle_drag_event_0(#keyboard{unicode=C}=Ev,
 	    wings_wm:dirty(),
 	    wings_wm:message_right(ModeFun(help, ModeData)),
 	    Val = {ModeData,Drag0#drag.falloff},
-	    Drag1 = case wings_pref:get_value(drag_resets) of
+	    Drag1 = case wings_pref:get_value(drag_resets) of %% Probably Remove This %%         
 	        false -> parameter_update(new_mode_data, Val,
-	                 Drag0#drag{mode_data=ModeData});
+	                 Drag0#drag{mode_data={changed,ModeData}});
 	        true -> parameter_update(new_mode_data, Val,
-	                Drag0#drag{mode_data=ModeData,xs=0,ys=0})
+	                Drag0#drag{mode_data=ModeData,xs=0,ys=0,zs=0,fp=0,
+	                           psum=[0,0,0,0]})
 	    end,
 	    Drag = case ModeFun(units, ModeData) of
 		       none -> Drag1;
 		       Units ->
-		           UnitSc = unit_scales(Units),
-		           Drag1#drag{unit=Units,unit_sc=UnitSc}
+		           Us = unit_scales(Units),
+		           #drag{xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,psum=Psum0,
+		                 unit_sc=Us0} = Drag1,
+		           Psum = adjust_unit_scaling(Psum0,Us,Us0),
+		           [Xs,Ys,Zs,Fp] = adjust_unit_scaling([Xs0,Ys0,Zs0,Fp0],Us,Us0),
+		           Drag1#drag{xs=Xs,ys=Ys,zs=Zs,fp=Fp,psum=Psum,unit=Units,
+		                      unit_sc=Us}
 		   end,
 	    get_drag_event(Drag)
     end;
@@ -510,11 +563,7 @@ handle_drag_event_1({drag_arguments,Move}, Drag0) ->
     DragEnded = {new_state,St#st{drag_args=Move}},
     wings_wm:later(DragEnded),
     pop;
-handle_drag_event_1(#mousebutton{button=3,state=?SDL_RELEASED}, Drag) ->
-    wings_dl:map(fun invalidate_fun/2, []),
-    ungrab(Drag),
-    wings_wm:later(revert_state),
-    pop;
+
 handle_drag_event_1(view_changed, Drag) ->
     get_drag_event(view_changed(Drag));
 handle_drag_event_1({action,{drag_arguments,_}=DragArgs}, _) ->
@@ -560,7 +609,10 @@ invalidate_fun(#dlo{src_we=We}=D, _) ->
 numeric_input(Drag0) ->
     {_,X,Y} = wings_wm:local_mouse_state(),
     Ev = #mousemotion{x=X,y=Y,state=0,mod=0},
-    {Move0,Drag} = mouse_translate(Ev, Drag0),
+    {Move0,Drag} = case mouse_translate(Ev, Drag0) of
+	    {{_,M},D} -> {M,D};
+		Other -> Other
+	end,
     wings_ask:dialog(?__(1,"Numeric Input"),
 		     make_query(Move0, Drag),
 		     fun(Res) ->
@@ -579,6 +631,8 @@ make_query_1([U0|Units], [V|Vals]) ->
 	    [{hframe,[{label,?__(2,"A")},
 		      {text,V,qrange(U0)},{label,[?DEGREE]}]}|
 	     make_query_1(Units, Vals)];
+	skip ->
+	     make_query_1(Units, Vals);
 	U ->
 	    [{hframe,[{label,qstr(U)},{text,V,qrange(U0)}]}|
 	     make_query_1(Units, Vals)]
@@ -591,7 +645,7 @@ qstr(dy) -> ?__(3,"Dy");
 qstr(dz) ->  ?__(4,"Dz");
 qstr(falloff) ->  ?__(5,"R");
 qstr(angle) ->	?__(6,"A");
-qstr(Atom) -> atom_to_list(Atom).
+qstr(Atom) -> wings_util:stringify(Atom).
 
 qrange({_,{_,_}=Range}) -> [{range,Range}];
 qrange(_) -> [].
@@ -609,6 +663,8 @@ safe_mul_100(A) ->
 make_move(Move, #drag{unit=Units}) ->
     make_move_1(Units, Move).
 
+make_move_1([skip|Units],Vals) ->
+    [0|make_move_1(Units, Vals)];
 make_move_1([{percent,_}=Unit|Units], [V|Vals]) ->
     [clamp(Unit, V/100)|make_move_1(Units, Vals)];
 make_move_1([percent|Units], [V|Vals]) ->
@@ -641,7 +697,7 @@ view_changed(#drag{flags=Flags}=Drag0) ->
 	      false ->
 	    wings_dl:map(fun view_changed_fun/2, []),
 	    {_,X,Y} = sdl_mouse:getMouseState(),
-	        Drag0#drag{x=X,y=Y,xs=0,ys=0,zs=0,fp=0}
+	        Drag0#drag{x=X,y=Y,xs=0,ys=0,zs=0,fp=0,psum=[0,0,0,0]}
 	    end
     end.
 
@@ -666,97 +722,118 @@ motion(Event, Drag0) ->
 
 mouse_translate(Event0, Drag0) ->
     Mode = wings_pref:get_value(camera_mode),
-    {Event,Mod} = mouse_pre_translate(Mode, Event0),
-    {Ds0,Drag} = mouse_range(Event, Drag0, Mod),
-    Ds = add_offset(Ds0, Drag),
-    Move = constrain(Ds, Mod, Drag),
-    {Move,Drag}.
+    {Event,Mod,Drag1} = mouse_pre_translate(Mode, Event0,Drag0),
+	mouse_range(Event, Drag1, Mod).
 
-add_offset(Ds, #drag{offset=Offset}) ->
-    add_offset_1(Ds, Offset).
-
-add_offset_1([D|Ds], [O|Ofs]) ->
-    [D+O|add_offset_1(Ds, Ofs)];
-add_offset_1([], _) -> [].
-
-mouse_pre_translate(_, #mousemotion{state=Mask,mod=Mod}=Ev) ->
+mouse_pre_translate(Mode, #mousemotion{state=Mask,mod=Mod}=Ev,Drag)
+        when Mode==blender; Mode==sketchup; Mode==tds ->
     if
-	Mask band ?SDL_BUTTON_RMASK =/= 0,
-	Mod band ?CTRL_BITS =/= 0 ->
-	    {Ev#mousemotion{state=?SDL_BUTTON_MMASK},
-	     Mod band (bnot ?CTRL_BITS)};
-	true -> {Ev,Mod}
-    end.
+    Mask band ?SDL_BUTTON_RMASK =/= 0,
+    Mod band ?CTRL_BITS =/= 0 ->
+        {Ev#mousemotion{state=?SDL_BUTTON_MMASK},
+         Mod band (bnot ?CTRL_BITS),Drag};
+    true -> {Ev,Mod,Drag}
+    end;
 
-mouse_range(#mousemotion{x=X0,y=Y0,state=Mask},
-	    #drag{x=OX,y=OY,xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,
-		  xt=Xt0,yt=Yt0,mmb_count=Count0,fp_count=FpCount0,
-		  unit_sc=UnitScales,unit=Unit}=Drag,
-	    Mod) ->
+mouse_pre_translate(_, #mousemotion{mod=Mod}=Ev,Drag) ->
+    {Ev,Mod,Drag}.
+
+mouse_range(#mousemotion{x=X0, y=Y0, state=Mask},
+            #drag{x=OX, y=OY,
+                   xs=Xs0, ys=Ys0, zs=Zs0, fp=Fp0,
+                   psum=Psum0,
+                   mode_data=MD,
+                   xt=Xt0, yt=Yt0, mmb_count=Count0,
+                   unit_sc=UnitScales, unit=Unit, offset=Offset,
+                   last_move=LastMove}=Drag0,
+			Mod) ->
     %%io:format("Mouse Range ~p ~p~n", [{X0,Y0}, {OX,OY,Xs0,Ys0}]),
+    [Xp,Yp,Zp,Fpp] = case Mod =/= 0 of
+        true -> Psum0;
+        false -> [Xs0,Ys0,Zs0,Fp0]
+    end,
     {X,Y} = wings_wm:local2global(X0, Y0),
-    case wings_pref:lowpass(X- OX, Y-OY) of
-	{0,0} ->
-	    {mouse_scale([Xs0,-Ys0,-Zs0,-Fp0], UnitScales),
-	     Drag#drag{xt=0,yt=0}};
-	{XD0,YD0} ->
-	    CS = constraints_scale(Unit,Mod,UnitScales),
-	    XD = CS*(XD0 + Xt0),
-	    YD = CS*(YD0 + Yt0),
-		case {Mask band ?SDL_BUTTON_MMASK =/= 0,Mask band ?SDL_BUTTON_RMASK =/= 0} of
-		  {true,false} ->
-		    Xs = Xs0,
-		    Ys = Ys0,
-		    Zs = case wings_pref:get_value(camera_mode) of
-		         maya -> Zs0 - XD;	%Horizontal motion
-		         _ -> Zs0 + YD	%Vertical motion
-		     end,
-		    Fp = Fp0,
-		    FpCount = FpCount0,
-		    Count = Count0 + 1;
-		  {false,true} ->
-		    Xs = Xs0,
-		    Ys = Ys0,
-		    Zs = Zs0,
-		    Fp = Fp0 + YD,
-		    FpCount = FpCount0 +1,
-		    Count = Count0;
-		  {_,_} ->
-		    Xs = Xs0 + XD,
-		    Ys = Ys0 + YD,
-		    Zs = Zs0,
-		    Fp = Fp0,
-		    FpCount = FpCount0,
-		    Count = Count0
-	    end,
-	    wings_io:warp(OX, OY),
-	    {mouse_scale([Xs,-Ys,-Zs,-Fp], UnitScales),
-	     Drag#drag{xs=Xs,ys=Ys,zs=Zs,fp=Fp,xt=XD0,yt=YD0,mmb_count=Count,fp_count=FpCount}}
+
+    case wings_pref:lowpass(X-OX, Y-OY) of
+    {0,0} ->
+        Drag = Drag0#drag{xt=0,yt=0},
+        {{no_change,LastMove},Drag};
+
+    {XD0,YD0} ->
+
+        CS = constraints_scale(Unit,Mod,UnitScales),
+        XD = CS*(XD0 + Xt0),
+        YD = CS*(YD0 + Yt0),
+
+        ModeData = case MD of
+          {_,MD0} -> MD0;
+          MD0 -> MD0
+        end,
+
+        case {Mask band ?SDL_BUTTON_MMASK =/= 0,Mask band ?SDL_BUTTON_RMASK =/= 0} of
+          {true,false} ->
+            Xs = {no_con, Xs0},
+            Ys = {no_con, -Ys0},
+            Zs = case wings_pref:get_value(camera_mode) of
+                   maya -> {con, - (Zp - XD)};	%Horizontal motion
+                   _cam -> {con, - (Zp + YD)}	%Vertical motion
+                 end,
+            Fp = {no_con, -Fp0},
+            Count = Count0 + 1;
+          {false,true} ->
+            Xs = {no_con, Xs0},
+            Ys = {no_con, -Ys0},
+            Zs = {no_con, -Zs0},
+            Fp = {con, - (Fpp + XD)},
+            Count = Count0;
+          {_,_} ->
+            Xs = {con, Xp + XD},
+            Ys = {con, - (Yp + YD)},
+            Zs = {no_con, -Zs0},
+            Fp = {no_con, -Fp0},
+            Count = Count0
+        end,
+        wings_io:warp(OX, OY),
+
+        % Ds means DragSummary
+        Ds0 = mouse_scale([Xs,Ys,Zs,Fp], UnitScales),
+        Ds1 = add_offset_to_drag_sum(Ds0, Unit, Offset),
+        Ds = round_to_constraint(Unit, Ds1, Mod, []),
+
+        Psum = [S || {_,S} <- [Xs,Ys,Zs,Fp]],
+        [Xs2,Ys2,Zs2,Fp2] = constrain_2(Unit, Psum, UnitScales, Offset),
+
+        [Xs1,Ys1,Zs1,Fp1] = scale_mouse_back(Ds, UnitScales, Offset),
+        Move = constrain_1(Unit, Ds, Drag0),
+        Drag = Drag0#drag{xs=Xs1,ys=-Ys1,zs=-Zs1,fp=-Fp1,
+                          psum=[Xs2,-Ys2,-Zs2,-Fp2],
+                          xt=XD0,yt=YD0,mmb_count=Count,mode_data=ModeData},
+        {Move,Drag}
     end.
 
-mouse_scale([D|Ds], [S|Ss]) ->
-    [D*S|mouse_scale(Ds, Ss)];
+mouse_scale([{Tag,D}|Ds], [S|Ss]) ->
+    [{Tag,D*S}|mouse_scale(Ds, Ss)];
 mouse_scale(Ds, _) -> Ds.
 
+scale_mouse_back([D|Ds], [S|Ss], [O|Ofs]) ->
+    [(D-O)/S|scale_mouse_back(Ds,Ss,Ofs)];
+scale_mouse_back(Ds, _, _) -> Ds.
+
 constraints_scale([U0|_],Mod,[UnitScales|_]) ->
     case wings_pref:get_value(con_alternate) of
     true -> case constraint_factor_alt(clean_unit(U0),Mod) of
-    none -> 1.0;
-    {_,What} ->
-        What*0.01/UnitScales
-            end;
+        none -> 1.0;
+        {_,What} ->
+          What*0.01/UnitScales
+      end;
     false -> case constraint_factor(clean_unit(U0),Mod) of
-            none -> 1.0;
-            {_,What} ->
-            What*0.01/UnitScales
-            end
+        none -> 1.0;
+        {_,What} ->
+          What*0.01/UnitScales
+      end
     end.
 
-constrain(Ds0, Mod, #drag{unit=Unit}=Drag) ->
-    Ds = constrain_0(Unit, Ds0, Mod, []),
-    constrain_1(Unit, Ds, Drag).
-
-constrain_0([U0|Us], [D0|Ds], Mod, Acc) ->
+round_to_constraint([U0|Us], [{con,D0}|Ds], Mod, Acc) ->
     U = clean_unit(U0),
     D = case wings_pref:get_value(con_alternate) of
       true -> case constraint_factor_alt(U, Mod) of
@@ -765,25 +842,53 @@ constrain_0([U0|Us], [D0|Ds], Mod, Acc) ->
           round(D0*F1)*F2
         end;
       false -> case constraint_factor(U, Mod) of
-	    none -> D0;
-	    {F1,F2} ->
-	      round(D0*F1)*F2
+        none -> D0;
+        {F1,F2} ->
+          round(D0*F1)*F2
             end
     end,
-    constrain_0(Us, Ds, Mod, [D|Acc]);
-constrain_0([_|_], [], _, Acc) -> reverse(Acc);
-constrain_0([], Ds, _, Acc) -> reverse(Acc, Ds).
+    round_to_constraint(Us, Ds, Mod, [D|Acc]);
+
+round_to_constraint([_|Us], [{no_con, D}|Ds], Mod, Acc) ->
+    round_to_constraint(Us, Ds, Mod, [D|Acc]);
+round_to_constraint([], [{_,D}|Ds], Mod, Acc) ->
+    round_to_constraint([], Ds, Mod, [D|Acc]);
+
+round_to_constraint([_|_], [], _, Acc) -> reverse(Acc);
+round_to_constraint([], [], _, Acc) -> reverse(Acc).
 
 constrain_1([falloff], _, #drag{falloff=Falloff}) ->
     [Falloff];
-constrain_1([U|Us], [D|Ds], Drag) ->
-    [clamp(U, D)|constrain_1(Us, Ds, Drag)];
+constrain_1([_|Us], [D|Ds], Drag) ->
+    [D|constrain_1(Us, Ds, Drag)];
 constrain_1([], _, _) -> [].
 
+constrain_2([U|Us], [D|Ds], [S|Ss], [O|Of]) ->
+    [clamp_2(U, D*S+O, S, O, D)|constrain_2(Us, Ds, Ss, Of)];
+constrain_2([], [D|Ds], [], Of) ->
+    [D|constrain_2([], Ds, [], Of)];
+constrain_2([],[],[],_) -> [].
+
+add_offset_to_drag_sum([{Tag,D}|Ds], [U|Us], [O|Ofs]) ->
+    [{Tag,clamp(U, D+O)}|add_offset_to_drag_sum(Ds, Us, Ofs)];
+add_offset_to_drag_sum([{Tag,D}|Ds], _, _) ->
+    [{Tag,D}|add_offset_to_drag_sum(Ds, [], [])];
+add_offset_to_drag_sum([],_,_) -> [].
+
+add_offset([D|Ds], [U|Us], [O|Ofs]) ->
+    [clamp(U, D+O)|add_offset(Ds, Us, Ofs)];
+add_offset([D|Ds], _, _) ->
+    [D|add_offset(Ds, [], [])];
+add_offset([],_,_) -> [].
+
 clamp({_,{Min,_Max}}, D) when D < Min -> Min;
 clamp({_,{_Min,Max}}, D) when D > Max -> Max;
 clamp(_, D) -> D.
 
+clamp_2({_,{Min,_Max}}, DSc, S, O, _) when DSc < Min -> (Min-O)/S;
+clamp_2({_,{_Min,Max}}, DSc, S, O, _) when DSc > Max -> (Max-O)/S;
+clamp_2(_, _, _, _, D) -> D.
+
 constraint_factor(angle, Mod) ->
     RCS = wings_pref:get_value(con_rot_shift),
     RCC = wings_pref:get_value(con_rot_ctrl),
@@ -932,6 +1037,20 @@ filter_angle(Degrees) ->
 %%%
 %%% Update selection for new mouse position.
 %%%
+motion_update({_,undefined}, #drag{unit=Units,offset=Offset}=Drag) ->
+    Move0 = lists:duplicate(length(Units),0.0),
+    Move1 = constrain_1(Units, Move0, Drag),
+    Move = add_offset(Move1, Units, Offset),
+    motion_update(Move,Drag);
+
+motion_update({no_change,LastMove}, #drag{unit=Units,mode_data={_,MD}}=Drag) ->
+    Move = constrain_1(Units, LastMove, Drag),
+    motion_update(Move,Drag#drag{mode_data=MD});
+motion_update({no_change,_}, #drag{falloff=none}=Drag) ->
+    Drag;
+motion_update({no_change,LastMove}, #drag{unit=Units}=Drag) ->
+    Move = constrain_1(Units, LastMove, Drag),
+    motion_update(Move,Drag);
 
 motion_update(Move, #drag{unit=Units}=Drag) ->
     wings_dl:map(fun(D, _) ->
@@ -1002,9 +1121,11 @@ unit(percent, P) ->
     trim(io_lib:format("~.2f%  ", [P*100.0]));
 unit(falloff, R) ->
     ["R: "|trim(io_lib:format("~10.2f", [R]))];
-unit(Unit, Move) ->
-    io:format("~p\n", [{Unit,Move}]),
-    [].
+unit(skip,_) ->
+    %% the atom 'skip' can be used as a place holder. See wpc_arc.erl
+    [];
+unit(Unit, D) ->
+    [wings_util:stringify(Unit) ++ ": "|trim(io_lib:format("~10.4f  ", [D]))].
 
 trim([$\s|T]) -> trim(T);
 trim([[_|_]=H|T]) ->
