@@ -3,12 +3,12 @@
 %%
 %%     This module contains most edge command and edge utility functions.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson.
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_edge.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_edge).
@@ -25,8 +25,7 @@
 -export([dissolve_isolated_vs/2]).
 
 -include("wings.hrl").
--import(lists, [foldl/3,last/1,member/2,reverse/1,reverse/2,
-		seq/2,sort/1]).
+-import(lists, [foldl/3,sort/1]).
 
 from_vs(Vs, We) when is_list(Vs) ->
     from_vs(Vs, We, []);
@@ -103,21 +102,17 @@ fast_cut(Edge, Pos0, We0) ->
     BColOther = get_vtx_color(NextBCol, Rf, Etab0),
     Weight = if
 		 Pos0 == default -> 0.5;
+		 VstartPos == VendPos -> 0.5;
+		 Pos0 == VstartPos -> 0.0;
+		 Pos0 == VendPos -> 1.0;
 		 true ->
 		     ADist = e3d_vec:dist(Pos0, VstartPos),
 		     BDist = e3d_vec:dist(Pos0, VendPos),
-%% 		     try ADist/(ADist+BDist)
-%% 		     catch
-%% 			 error:badarith -> 0.5
-%% 		     end
-		     case catch ADist/(ADist+BDist) of
-			 {'EXIT',_} -> 0.5;
-			 Else -> Else
-		     end
+		     ADist/(ADist+BDist)
 	     end,
     NewColA = wings_color:mix(Weight, AColOther, ACol),
     NewColB = wings_color:mix(Weight, BCol, BColOther),
-    
+
     NewEdgeRec = Template#edge{vs=NewV,a=NewColA,ltsu=Edge,rtpr=Edge},
     Etab1 = gb_trees:insert(NewEdge, NewEdgeRec, Etab0),
     Etab2 = patch_edge(EdgeA, NewEdge, Edge, Etab1),
