@@ -8,14 +8,14 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_extrude_face.erl 207 2008-01-04 16:18:30Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_extrude_face).
 -export([faces/2,regions/2]).
 
 -include("wings.hrl").
--import(lists, [foldl/3,foreach/2,last/1,reverse/1,reverse/2,sort/1,merge/1]).
+-import(lists, [foldl/3,foreach/2,last/1,reverse/2,sort/1,merge/1]).
 
 %%%
 %%% Extrusion of faces individually (used by Extrude, Inset, Bevel).
