@@ -3,12 +3,12 @@
 %%
 %%     This module contains most of the face commands.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_face_cmd.erl 202 2008-01-04 09:20:30Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_face_cmd).
@@ -16,15 +16,11 @@
 -export([mirror_faces/2,set_color/2,force_bridge/5]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldl/3,reverse/1,reverse/2,sort/1,keysort/2,
-		keymember/3,keysearch/3,keydelete/3,
-		member/2,seq/2,last/1]).
+-import(lists, [foldl/3,reverse/1,sort/1,member/2]).
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
-    Menu = [{basic,{?__(1,"Face operations"),ignore}},
-	    {basic,separator},
-	    {?__(2,"Move"),{move,Dir},[],[magnet]},
+    Menu = [{?__(2,"Move"),{move,Dir},[],[magnet]},
 	    wings_menu_util:rotate(St),
 	    wings_menu_util:scale(St),
 	    separator,
@@ -34,27 +30,26 @@ menu(X, Y, St) ->
 	    separator,
 	    wings_menu_util:flatten(),
 	    separator,
-	    {?__(6,"Inset"),inset,
-	     ?__(7,"Inset a face inside the selected face")},
 	    {?__(8,"Intrude"),intrude,
 	     ?__(9,"Carve out interior of object, making selected faces holes")},
 	    {?__(10,"Bevel"),bevel,
 	     ?__(11,"Round off edges of selected faces")},
 	    {?__(12,"Bridge"),bridge,
 	     ?__(13,"Create a bridge or tunnel between two faces")},
-	    {advanced,separator},
+	    separator,
 	    {?__(14,"Bump"),bump,
 	     ?__(15,"Create bump of selected faces")},
-	    {advanced,{?__(16,"Lift"),{lift,lift_fun(St)}}},
-	    {advanced,{?__(17,"Put On"),put_on_fun(),
-		       {?__(18,"Move and rotate object, aligning the selected face to another element"),[],
-		        ?__(19,"Clone object on to one or more elements")},[]}},
+	    {?__(16,"Lift"),{lift,lift_fun(St)}},
+	    {?__(17,"Put On"),put_on_fun(),
+	     {?__(18,"Move and rotate object, aligning the selected face to another element"),[],
+	      ?__(19,"Clone object on to one or more elements")},[]},
 	    separator,
 	    {?__(20,"Mirror"),mirror_fun(),
 	     {?__(21,"Mirror object around selected faces and merge to object"),[],
 	      ?__(22,"Mirror and create separate objects")},[]},
-    	    {?__(23,"Dissolve"),dissolve,
-	     ?__(24,"Eliminate all edges between selected faces")},
+	    {?__(23,"Dissolve"),dissolve_fun(),
+	     {?__(24,"Eliminate all edges between selected faces"),[],
+		  ?__(34,"Eliminate selected faces and remove remaining isolated verts")},[]},
 	    {?__(25,"Collapse"),collapse,
 	     ?__(26,"Delete faces, replacing them with vertices")},
 	    {?__(27,"Smooth"),smooth,
@@ -67,6 +62,13 @@ menu(X, Y, St) ->
 	  ?__(31,"Apply vertex colors to selected faces")}],
     wings_menu:popup_menu(X, Y, face, Menu).
 
+dissolve_fun() ->
+    fun
+	(1, _Ns) -> {face,dissolve};
+	(3, _Ns) -> {face,clean_dissolve};
+	(_, _) -> ignore
+    end.
+
 lift_fun(St) ->
     fun(help, _Ns) ->
 	    {?__(1,"Lift, rotating face around edge or vertex"),[],
@@ -107,8 +109,6 @@ command({flatten,Plane}, St) ->
     flatten(Plane, St);
 command(bevel, St) ->
     ?SLOW(wings_extrude_edge:bevel_faces(St));
-command(inset, St) ->
-    ?SLOW(inset(St));
 command(mirror, St) ->
     ?SLOW({save_state,mirror(St)});
 command(mirror_separate, St) ->
@@ -117,6 +117,8 @@ command(intrude, St) ->
     ?SLOW(intrude(St));
 command(dissolve, St) ->
     {save_state,dissolve(St)};
+command(clean_dissolve, St) ->
+    {save_state,clean_dissolve(St)};
 command(bridge, St) ->
     {save_state,bridge(St)};
 command(smooth, St) ->
@@ -156,9 +158,6 @@ command(hide, St) ->
 extrude(Type, St) ->
     wings_move:setup(Type, extrude_faces(St)).
 
-inset(St) ->
-    wings_scale:inset(extrude_faces(St)).
-
 extrude_faces(St) ->
     wings_sel:map(fun(Faces, We) ->
 			  wings_extrude_face:faces(Faces, We)
@@ -222,7 +221,7 @@ extract_region(Type, St0) ->
     Sel = St1#st.sel,
     St = wings_sel:set(Sel, St1),
     wings_move:setup(Type, St).
-    
+
 %%%
 %%% The Dissolve command.
 %%%
@@ -236,9 +235,26 @@ dissolve_sel(Faces, #we{id=Id}=We0, Acc) ->
     Sel = wings_we:new_items_as_gbset(face, We0, We),
     {We,[{Id,Sel}|Acc]}.
 
+%%%
+%%% Clean Dissolve
+%%%
+
+clean_dissolve(St0) ->
+    {St,Sel} = wings_sel:mapfold(fun clean_dissolve_sel/3, [], St0),
+    wings_sel:set(Sel, St).
+
+clean_dissolve_sel(Faces, #we{id=Id}=We0, Acc) ->
+    IsolatedVs1 = wings_vertex:isolated(We0),
+    We1 = wings_dissolve:faces(Faces, We0),
+    IsolatedVs2 = wings_vertex:isolated(We1),
+    C = IsolatedVs2 -- IsolatedVs1,
+    We = wings_edge:dissolve_isolated_vs(C, We1),
+    Sel = wings_we:new_items_as_gbset(face, We0, We),
+    {We,[{Id,Sel}|Acc]}.
+
 %%%
 %%% The Intrude command.
-%%% 
+%%%
 
 intrude(St0) ->
     St1 = dissolve(St0),
