@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_hotkey.erl 223 2008-01-09 21:36:03Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_hotkey).
@@ -19,7 +19,7 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 
--import(lists, [foldl/3,sort/1,foreach/2,map/2,last/1,reverse/1]).
+-import(lists, [foldl/3,sort/1,foreach/2,map/2,reverse/1]).
 
 -compile({parse_transform,ms_transform}).
 
@@ -41,11 +41,26 @@ event(Ev, #st{selmode=Mode}=St) ->
     end;
 event(Ev, Cmd) -> event_1(Ev, Cmd).
 
-event_1(#keyboard{}=Ev, SelMode) ->
-    case lookup(Ev, SelMode) of
-	next -> lookup(Ev, none);
+event_1(#keyboard{}=Ev, {Selmode,_}=Cmd)
+  when Selmode==vertex; Selmode==edge; Selmode==face; Selmode==body; Selmode==light ->
+    case lookup(Ev, Cmd) of
+	next -> event_1(Ev,Selmode);
 	Action -> Action
     end;
+event_1(#keyboard{}=Ev, Selmode)
+  when is_atom(Selmode) ->
+    case lookup(Ev, Selmode) of
+	next -> lookup_1(Ev,Selmode);
+	Action -> Action
+    end;
+
+event_1(#keyboard{}=Ev, Cmd) when Cmd =/= none ->
+    Mode = check_for_mode_specific_menubar_items(Cmd),
+    case lookup(Ev, Cmd) of
+      next when is_atom(Mode) -> event_1(Ev,Mode);
+      next -> lookup(Ev,none);
+      Action -> Action
+    end;
 event_1(_, _) -> next.
 
 lookup(Ev, Cmd) ->
@@ -54,6 +69,31 @@ lookup(Ev, Cmd) ->
 	[] -> next
     end.
 
+lookup_1(Ev,SelMode) ->
+%% Checks for menubar items that are selection mode specific.
+    case lookup(Ev, none) of
+      {select,{edge_loop,edge_loop}}=EL when SelMode == face ->
+          EL;
+      {select,{edge_loop,_}} when SelMode =/= edge ->
+          next;
+      {select,{oriented_faces,_}} when SelMode =/= face ->
+          next;
+      {select,{similar_material,_}} when SelMode =/= face ->
+          next;
+      {select,{similar_area,_}} when SelMode =/= face ->
+          next;
+      {tools,{virtual_mirror,create}} when SelMode =/= face ->
+          next;
+      Other -> Other
+    end.
+
+check_for_mode_specific_menubar_items({select,{edge_loop,_}}) -> edge;
+check_for_mode_specific_menubar_items({select,{oriented_faces,_}}) -> face;
+check_for_mode_specific_menubar_items({select,{similar_material,_}}) -> face;
+check_for_mode_specific_menubar_items({select,{similar_area,_}}) -> face;
+check_for_mode_specific_menubar_items({tools,{virtual_mirror,create}}) -> face;
+check_for_mode_specific_menubar_items(Cmd) -> Cmd.
+
 %%%
 %%% Binding and unbinding of keys.
 %%%
@@ -318,13 +358,13 @@ default_keybindings() ->
      {{$z,[ctrl]},          {edit,undo_toggle}},
      {{?SDLK_KP_PLUS,[]},   {select,more}},
      {{?SDLK_KP_MINUS,[]},  {select,less}},
-     {{?SDLK_F3,[]},        {select,{edge_loop,prev_edge_loop}}},
-     {{?SDLK_F4,[]},        {select,{edge_loop,next_edge_loop}}},
+     {{?SDLK_F6,[]},        {select,{edge_loop,prev_edge_loop}}},
+     {{?SDLK_F7,[]},        {select,{edge_loop,next_edge_loop}}},
      {{?SDLK_F5,[]},        {select,{by,{faces_with,5}}}},
      {{?SDLK_TAB,[]},       {view,workmode}},
      {{?SDLK_TAB,[shift]},  {view,quick_preview}},
      {$\s,              {select,deselect}},
-     {$a,               {view,aim}},
+     {$a,               {view,highlight_aim}},
      {$A,               {view,frame}},
      {$b,               {select,body}},
      {{$d,[ctrl]},      {edit,repeat}},
