@@ -3,12 +3,12 @@
 %%
 %%     This module manages the face materials (i.e. colors and textures).
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_material.erl 283 2008-03-09 01:24:24Z optigon $
+%%     $Id$
 %%
 
 -module(wings_material).
@@ -24,16 +24,11 @@
 -include("wings.hrl").
 -include("e3d_image.hrl").
 
--import(lists, [map/2,foreach/2,sort/1,foldl/3,reverse/1,reverse/2,
+-import(lists, [sort/1,foldl/3,reverse/1,
 		keyreplace/4,keydelete/3,keysearch/3,flatten/1]).
 
 material_menu(St) ->
-    [{basic,{?__(1,"Material"),
-	     {material,
-	      [{?__(2,"New..."),new,
-		?__(3,"Create a new material and assign to selected faces")},
-	       separator|mat_list(St)]}}},
-     {advanced,{?__(4,"Material"),{material,material_fun(St)}}}].
+    [{?__(4,"Material"),{material,material_fun(St)}}].
 
 material_fun(St) ->
     fun(help, _Ns) ->
@@ -192,12 +187,13 @@ select_material(Mat, St) ->
     %% XXX Works but is slow.
     wings_sel:make(fun(_, #we{mat=AtomMat}) when is_atom(AtomMat) ->
 			   Mat =:= AtomMat;
-		      (Face, #we{mat=MatTab}) ->
+		      (Face, #we{mat=MatTab}) when Face >= 0 ->
 			   case keysearch(Face, 1, MatTab) of
 			       false -> false;
 			       {value,{_,Mat}} -> true;
 			       {value,_} -> false
-			   end
+			   end;
+			   (_, _) -> false
 		   end, face, St).
 
 set_material(Mat, #st{selmode=face}=St) ->
