@@ -3,12 +3,12 @@
 %%
 %%     Implementation of pulldown and popup menus.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_menu.erl 279 2008-03-08 09:09:47Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_menu).
@@ -24,25 +24,31 @@
 -define(SEPARATOR_HEIGHT, (wings_text:height()-4)).
 -define(INITIAL_LEVEL, 1).
 
-%% Menu information kept for a popup menu.
+-type unicode_char() :: 0..2097151.
+-type unicode_string() :: list(unicode_char()).
+
+-type menu_help() :: unicode_string() |
+  {unicode_string(),unicode_string(),unicode_string()}.
+
+-type menu_item() :: {unicode_string(),any(),menu_help(),unicode_string(),list()} |
+  'separator'.
+
+%% Menu information kept for menus.
 -record(mi,
-	{ymarg,					%Margin at top and bottom
-	 shortcut,				%Position for shortcut (pixels)
-	 w,					%Width of menu (pixels)
-	 h,					%Height of menu (pixels)
-	 hs,					%Tuple: height of each entry.
-	 sel=none,				%Selected item (1..tuple_size(Menu))
-	 sel_side=left,				%Selection on left or right.
+	{ymarg :: non_neg_integer(),		%Margin at top and bottom
+	 shortcut :: non_neg_integer(),		%Position for shortcut (pixels)
+	 w :: pos_integer(),			%Width of menu (pixels)
+	 h :: pos_integer(),			%Height of menu (pixels)
+	 hs :: list(pos_integer()),		%Height of each entry.
+	 sel=none :: 'none'|pos_integer(),	%Selected item (1..tuple_size(Menu))
+	 sel_side=left :: 'left'|'right',	%Selection on left or right.
 	 ns=[],					%Name stack.
-	 menu,					%Original menu term
+	 menu :: tuple(menu_item()),		%Normalized menu.
 	 timer=make_ref(),			%Active submenu timer.
 	 level=?INITIAL_LEVEL,			%Menu level.
-	 adv,					%Advanced menus (true|false).
-	 ignore_rel=true,			%Ignore button release if
-						% just openened.
-	 type=plain,				%Type of menu: plain|popup
+	 type :: 'plain'|'popup',	        %Type of menu.
 	 owner,					%Owning window.
-	 flags=[]				%Flags (magnet/dialog).
+	 flags=[] :: list()			%Flags (magnet/dialog).
 	}).
 
 %%%
@@ -53,53 +59,46 @@
 %%%   {Text,Name,Hotkey,Help,Properties}
 %%%
 %%%   The Help field is normalized to
-%%%      String       for basic menu mode
-%%%      {L,M,R}      for advanced menu
+%%%      String       for plain (pull-down) menus
+%%%      {L,M,R}      for pop-up menus
 %%%
 
 is_popup_event(#mousebutton{button=3,x=X0,y=Y0,state=State,mod=Mod}) ->
     {X,Y} = wings_wm:local2global(X0, Y0),
-    case wings_pref:get_value(advanced_menus) of
-	true when State =:= ?SDL_RELEASED ->
-	    {yes,X,Y,Mod};
-	false when State =:= ?SDL_PRESSED  ->
+    case State of
+	?SDL_RELEASED ->
 	    {yes,X,Y,Mod};
 	_Other -> no
     end;
 is_popup_event(_Event) -> no.
 
 menu(X, Y, Owner, Name, Menu) ->
-    menu_setup(plain, X, Y, Name, Menu,
-	       #mi{adv=false,owner=Owner,ignore_rel=false}).
+    menu_setup(plain, X, Y, Name, Menu, #mi{owner=Owner}).
 
 popup_menu(X, Y, Name, Menu) ->
-    Adv = wings_pref:get_value(advanced_menus),
-    IgnoreRel = (Adv == false),
-    menu_setup(popup, X, Y, Name, Menu,
-	       #mi{adv=Adv,ignore_rel=IgnoreRel,owner=wings_wm:this()}).
+    menu_setup(popup, X, Y, Name, Menu, #mi{owner=wings_wm:this()}).
 
-menu_setup(Type, X0, Y0, Name, Menu0, #mi{ns=Names0,adv=Adv}=Mi0) ->
+menu_setup(Type, X0, Y0, Name, Menu0, #mi{ns=Names0}=Mi0) ->
     Names = [Name|Names0],
     Menu1 = wings_plugin:menu(list_to_tuple(reverse(Names)), Menu0),
     Hotkeys = wings_hotkey:matching(Names),
-    Menu = normalize_menu(Menu1, Hotkeys, Adv),
+    Menu = normalize_menu(Menu1, Hotkeys, Type =:= popup),
     {MwL,MwM,MwR,Hs} = menu_dims(Menu),
     Cw = wings_text:width(),
     TotalW = MwL + MwM + MwR + 8* Cw,
     Mh = lists:sum(Hs),
     Margin = 3,
+    InfoLine = ?CHAR_HEIGHT + 14,
     {X1,Y1} = case Type of
 		  plain ->
 		      {X0,Y0};
-		  popup when Adv == false ->
-		      {X0-TotalW div 2,Y0 - Margin div 2};
 		  popup ->
 		      {X0-TotalW div 2,Y0 - Margin - ?CHAR_HEIGHT}
 	      end,
-    {X,Y} = move_if_outside(X1, Y1, TotalW, Mh+2*Margin, Mi0),
+    {X,Y} = move_if_outside(X1, Y1, TotalW, Mh+2*Margin+InfoLine, Mi0),
     W = TotalW-10,
     Mi = Mi0#mi{ymarg=Margin,shortcut=MwL+Cw,w=TotalW-10,h=Mh,hs=Hs,
-		sel=none,ns=Names,menu=Menu,adv=Adv,type=Type},
+		sel=none,ns=Names,menu=Menu,type=Type},
     #mi{level=Level} = Mi,
     setup_menu_killer(Mi),
     Op = {seq,push,get_menu_event(Mi)},
@@ -176,24 +175,16 @@ menu_show(#mi{ymarg=Margin,shortcut=Shortcut,w=Mw,h=Mh}=Mi) ->
 normalize_menu(Menu, Hotkeys, Adv) ->
     normalize_menu(Menu, Hotkeys, Adv, []).
 
-normalize_menu([{basic,_}|Els], Hotkeys, true, Acc) ->
-    normalize_menu(Els, Hotkeys, true, Acc);
-normalize_menu([{basic,El}|Els], Hotkeys, false, Acc) ->
-    normalize_menu([El|Els], Hotkeys, false, Acc);
-normalize_menu([{advanced,_}|Els], Hotkeys, false, Acc) ->
-    normalize_menu(Els, Hotkeys, false, Acc);
-normalize_menu([{advanced,El}|Els], Hotkeys, true, Acc) ->
-    normalize_menu([El|Els], Hotkeys, true, Acc);
 normalize_menu([[_|_]=List|Els], Hotkeys, Adv, Acc) ->
     normalize_menu(List++Els, Hotkeys, Adv, Acc);
 normalize_menu([Elem0|Els], Hotkeys, Adv, Acc) ->
     Elem1 = case Elem0 of
 		{S,Name,Help,Ps} ->
-		    {S,Name,[],Help,Props=adv_filter(Adv, Ps)};
+		    {S,Name,[],Help,Props=Ps};
 		{S,Name,[C|_]=Help} when is_integer(C) ->
 		    {S,Name,[],Help,Props=[]};
 		{S,Name,Ps} ->
-		    {S,Name,[],[],Props=adv_filter(Adv, Ps)};
+		    {S,Name,[],[],Props=Ps};
 		{S,Name} ->
 		    {S,Name,[],[],Props=[]};
 		separator ->
@@ -206,10 +197,6 @@ normalize_menu([Elem0|Els], Hotkeys, Adv, Acc) ->
     normalize_menu(Els, Hotkeys, Adv, [Elem|Acc]);
 normalize_menu([], _Hotkeys, _Adv, Acc) -> list_to_tuple(reverse(Acc)).
 
-adv_filter(false, [magnet|T]) -> adv_filter(false, T);
-adv_filter(Flag, [H|T]) -> [H|adv_filter(Flag, T)];
-adv_filter(_, []) -> [].
-
 norm_add_hotkey(_, separator, _, _) -> separator;
 norm_add_hotkey(_, Elem, [], _) -> Elem;
 norm_add_hotkey({_,[_|_]}, Elem, _, _) -> Elem;
@@ -361,19 +348,17 @@ mousemotion(X, Y, Mi0) ->
     Mi = set_submenu_timer(Mi1, Mi0, X, Y),
     get_menu_event(Mi).
 
-button_pressed(#mousebutton{state=?SDL_RELEASED}, #mi{ignore_rel=true}=Mi) ->
-    get_menu_event(Mi#mi{ignore_rel=false});
 button_pressed(#mousebutton{button=B,x=X,y=Y,state=?SDL_RELEASED},
-	       #mi{adv=false}=Mi) when (B =< 3) ->
+	       #mi{type=plain}=Mi) when B =< 3 ->
     wings_wm:dirty(),
     button_pressed(1, 0, X, Y, Mi);
-button_pressed(#mousebutton{button=B,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Mi)
-  when (B =< 3) ->
+button_pressed(#mousebutton{button=B,x=X,y=Y,mod=Mod,state=?SDL_RELEASED},
+	       #mi{type=popup}=Mi) when B =< 3 ->
     wings_wm:dirty(),
     button_pressed(B, Mod, X, Y, Mi);
 button_pressed(_, _) -> keep.
 
-button_pressed(Button, Mod, X, Y, #mi{ns=Names,menu=Menu,adv=Adv}=Mi0) ->
+button_pressed(Button, Mod, X, Y, #mi{ns=Names,menu=Menu,type=Type}=Mi0) ->
     clear_timer(Mi0),
     Mi1 = update_highlight(X, Y, Mi0),
     Mi = update_flags(Mod, Mi1),
@@ -385,9 +370,9 @@ button_pressed(Button, Mod, X, Y, #mi{ns=Names,menu=Menu,adv=Adv}=Mi0) ->
 		{_,{'VALUE',Act0},_,_,Ps} ->
 		    Act = was_option_hit(Button, Act0, X, Ps, Mi),
 		    do_action(Act, Names, Ps, Mi);
-		{_,{Name,Submenu},_,_,_} when Adv == true ->
+		{_,{Name,Submenu},_,_,_} when Type =:= popup ->
 		    popup_submenu(Button, X, Y, Name, Submenu, Mi);
-		{_,{Name,Submenu},_,_,_} when Adv == false ->
+		{_,{Name,Submenu},_,_,_} when Type =:= plain ->
 		    submenu(Item, Name, Submenu, Mi);
 		{_,Act0,_,_,Ps} when is_function(Act0) ->
 		    call_action(Act0, Button, Names, Ps, Mi);
@@ -431,13 +416,17 @@ handle_key_1(cancel, _) ->
 handle_key_1(delete, Mi0) ->
     %% Delete hotkey bound to this entry.
     case current_command(Mi0) of
-	[] -> keep;
-	[_|_]=Cmds0 ->
-	    Cmds = [C || {_,C} <- Cmds0],
-	    case wings_hotkey:hotkeys_by_commands(Cmds) of
-		[] -> keep;			%No hotkeys for this entry.
-		Hotkeys -> hotkey_delete_dialog(Hotkeys)
-	    end
+      [] -> keep;
+      [_|_]=Cmds0 ->
+        Cmds = case Cmds0 of
+          [{1,{A,{B,false}}}] -> [{A,{B,false}},{A,{B,true}}];
+          [{1,{A,{B,{C,false}}}}] -> [{A,{B,{C,false}}},{A,{B,{C,true}}}];
+          _Cmd -> [C || {_,C} <- Cmds0]
+        end,
+        case wings_hotkey:hotkeys_by_commands(Cmds) of
+          [] -> keep;			%No hotkeys for this entry.
+          Hotkeys -> hotkey_delete_dialog(Hotkeys)
+        end
     end;
 handle_key_1(insert, Mi) ->
     %% Define new hotkey for this entry.
@@ -493,11 +482,11 @@ current_command(#mi{sel=Sel,menu=Menu,ns=Names,owner=Owner}=Mi)
     end;
 current_command(_) -> [].
 
-all_current_commands(Fun, #mi{adv=false}) ->
-    %% This menu is in basic mode, so we must only return the
-    %% command for LMB.
+all_current_commands(Fun, #mi{type=plain}) ->
+    %% This is a pull-down menu in plain mode, so we must
+    %% only return the command for LMB.
     all_current_commands_1([1], Fun);
-all_current_commands(Fun, #mi{adv=true}) ->
+all_current_commands(Fun, #mi{type=popup}) ->
     all_current_commands_1([1,2,3], Fun).
 
 all_current_commands_1([B|Bs], Fun) ->
@@ -597,8 +586,8 @@ insert_magnet_flags_0(Tuple0) when is_tuple(Tuple0) ->
     list_to_tuple(Tuple);
 insert_magnet_flags_0(Term) -> Term.
 
+%% Handle sub-menus of plain (pull-down) menus
 submenu(I, Name, Menu0, #mi{w=W,hs=Hs,level=Level}=Mi0) ->
-    %% Only in basic menu mode.
     Menu = expand_submenu(1, Name, Menu0, Mi0),
     X0 = W-?CHAR_WIDTH,
     Y0 = get_item_pos(I, Hs, -?LINE_HEIGHT),
@@ -675,7 +664,7 @@ update_highlight(X, Y, #mi{menu=Menu,sel=OldSel,sel_side=OldSide,w=W}=Mi0) ->
 	    Mi;
 	Item when is_integer(Item) ->
 	    wings_wm:dirty(),
-	    Mi = Mi0#mi{sel=Item,ignore_rel=false},
+	    Mi = Mi0#mi{sel=Item},
 	    help_text(Mi),
 	    Mi
     end.
@@ -686,10 +675,10 @@ was_option_hit(Button, Act, X, Ps, Mi) ->
 	true -> {Act,hit_right(Button, X, Mi)}
     end.
 
-hit_right(B, _, #mi{adv=true}) when B > 1 -> true;
+hit_right(B, _, #mi{type=popup}) when B > 1 -> true;
 hit_right(_, X, #mi{w=W}) -> X >= W-3*?CHAR_WIDTH.
 
-selected_item(Y, #mi{adv=Adv,ymarg=Margin,h=H,menu=Menu}=Mi) ->
+selected_item(Y, #mi{type=Type,ymarg=Margin,h=H,menu=Menu}=Mi) ->
     %% The tests are simplified because we know that the mouse cursor
     %% must be over the menu window.
     if
@@ -700,10 +689,10 @@ selected_item(Y, #mi{adv=Adv,ymarg=Margin,h=H,menu=Menu}=Mi) ->
 	    %% Above upper margin. If advanced menus, it is safe
 	    %% to count that as if the mouse is over the first row.
 	    %% (The menu doesn't popup until the RMB has been released,
-	    %% unlike the basic menus.)
-	    if
-		Adv -> selected_item_1(0, Mi);
-		true -> none
+	    %% unlike the plain (pull-down) menus.)
+	    case Type of
+		popup -> selected_item_1(0, Mi);
+		plain -> none
 	    end;
 	true ->
 	    %% Below the lower margin. Pretend that the cursor is over the
@@ -734,8 +723,8 @@ selected_item_1(Y0, I, [H|Hs], #mi{sel=OldSel,menu=Menu}=Mi) ->
 	Y -> selected_item_1(Y, I+1, Hs, Mi)
     end.
 
-is_submenu(_I, #mi{adv=true}) -> false;
-is_submenu(I, #mi{menu=Menu}) when is_integer(I) ->
+is_submenu(_I, #mi{type=popup}) -> false;
+is_submenu(I, #mi{type=plain,menu=Menu}) when is_integer(I) ->
     case element(I, Menu) of
 	separator -> false;
 	{_Text,{'VALUE',_},_Hotkey,_Help,_Ps} -> false;
@@ -748,10 +737,10 @@ build_command(Name, Names) ->
     foldl(fun(N, A) -> {N,A} end, Name, Names).
 
 menu_draw(_X, _Y, _Shortcut, _Mw, _I, [], _Mi) -> ok;
-menu_draw(X, Y, Shortcut, Mw, I, [H|Hs], #mi{menu=Menu,adv=Adv}=Mi) ->
+menu_draw(X, Y, Shortcut, Mw, I, [H|Hs], #mi{menu=Menu,type=Type}=Mi) ->
     ?CHECK_ERROR(),
     Elem = element(I, Menu),
-    Text = menu_text(Elem, Adv),
+    Text = menu_text(Elem, Type),
     case Elem of
 	separator -> draw_separator(X, Y, Mw);
 	{_,ignore,_,_,Ps} ->
@@ -774,7 +763,9 @@ menu_draw(X, Y, Shortcut, Mw, I, [H|Hs], #mi{menu=Menu,adv=Adv}=Mi) ->
 				wings_io:unclipped_text(X, Y, Text),
 				draw_hotkey(X, Y, Shortcut, Hotkey)
 			end),
-	    draw_submenu(Adv, Sub, X+Mw-5*?CHAR_WIDTH, Y-?CHAR_HEIGHT div 3);
+	    
+	    draw_submenu_marker(Type, Sub,
+				X+Mw-5*?CHAR_WIDTH, Y-?CHAR_HEIGHT div 3);
 	{_,_,Hotkey,_Help,Ps} ->
 	    menu_draw_1(Y, Ps, I, Mi,
 			fun() ->
@@ -826,8 +817,8 @@ menu_draw_1(_, _, _, _, DrawLeft, DrawRight) ->
 	_ -> DrawRight()
     end.
 
-menu_text({Text,{_,Fun},_,_,_}, true) when is_function(Fun) -> [$.,Text,$.];
-menu_text({Text,Fun,_,_,_}, true) when is_function(Fun) -> [$.,Text,$.];
+menu_text({Text,{_,Fun},_,_,_}, popup) when is_function(Fun) -> [$.,Text,$.];
+menu_text({Text,Fun,_,_,_}, popup) when is_function(Fun) -> [$.,Text,$.];
 menu_text({Text,_,_,_,_}, _) -> Text;
 menu_text(separator, _) -> [].
 
@@ -856,12 +847,11 @@ help_text(#mi{menu=Menu,sel=Sel}=Mi) ->
     Elem = element(Sel, Menu),
     help_text_1(Elem, Mi).
 
-help_text_1({Text,{Sub,_},_,_,_}, #mi{adv=false}) when Sub =/= 'VALUE' ->
-    %% No specific help text for submenus in basic mode.
+help_text_1({Text,{Sub,_},_,_,_}, #mi{type=plain}) when Sub =/= 'VALUE' ->
+    %% No specific help text for submenus in plain mode.
     Help = [Text|?__(1," submenu")],
     wings_wm:message(Help, "");
-help_text_1({_,{Name,Fun},_,_,Ps}, #mi{ns=Ns}=Mi)
-  when is_function(Fun) ->
+help_text_1({_,{Name,Fun},_,_,Ps}, #mi{ns=Ns}=Mi) when is_function(Fun) ->
     %% "Submenu" in advanced mode.
     Help0 = Fun(help, [Name|Ns]),
     Help = help_text_2(Help0),
@@ -915,8 +905,8 @@ draw_right_1(X0, Y0, Mw, Ps) ->
 	    wings_io:border(X, Y, Cw, Ch-1, Color)
     end.
 
-draw_submenu(true, _Item, _X, _Y) -> ok;
-draw_submenu(false, _Item, X, Y) ->
+draw_submenu_marker(popup, _Item, _X, _Y) -> ok;
+draw_submenu_marker(plain, _Item, X, Y) ->
     Cw = wings_text:width(),
     H = (wings_text:height()+2) div 3,
     ?CHECK_ERROR(),
@@ -981,10 +971,15 @@ have_magnet(Ps) ->
 
 get_hotkey([{1,Cmd}], Mi) ->
     wings_wm:dirty(),
-    wings_wm:message(hotkey_key_message(Cmd)),
-    {push,fun(Ev) ->
-		  handle_key_event(Ev, Cmd, Mi)
-	  end};
+    case Cmd of
+      {A,{B,false}} -> get_hotkey([{1,Cmd},{3,{A,{B,true}}}],Mi);
+      {A,{B,{C,false}}} -> get_hotkey([{1,Cmd},{3,{A,{B,{C,true}}}}],Mi);
+      Cmd ->
+        wings_wm:message(hotkey_key_message(Cmd)),
+        {push,fun(Ev) ->
+          handle_key_event(Ev, Cmd, Mi)
+          end}
+    end;
 get_hotkey([_|_]=Cmds, Mi) ->
     wings_wm:dirty(),
     wings_wm:message(hotkey_mouse_message(Cmds)),
