@@ -3,18 +3,18 @@
 %%
 %%     This module implements the Move command.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_move.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 -module(wings_move).
 -export([setup/2,setup_we/4,plus_minus/3,magnet_move_fun/3]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldr/3,foldl/3,sort/1]).
+-import(lists, [foldl/3,sort/1]).
 
 setup({'ASK',Ask}, St) ->
     wings:ask(Ask, St, fun setup/2);
@@ -46,7 +46,8 @@ plus_minus({'ASK',Ask}, Tvs, St0) ->
 plus_minus(Type, Tvs0, #st{selmode=Mode}=St) ->
     Vec = make_vector(Type),
     Tvs = plus_minus_2(Mode, Vec, Tvs0, []),
-    wings_drag:setup(Tvs, unit(Type, [falloff]), flags(Type), St).
+	Flags = [flags(Type),{initial,[0.0,0.0,0.0,1.0]}],
+    wings_drag:setup(Tvs, unit(Type, [{percent,{0.0,infinity}}]), Flags, St).
 
 plus_minus_2(Mode, Vec, [{Items,NewVs,Forbidden,We}|T], Acc0) ->
     Tv = setup_we(Mode, Vec, Items, We),
@@ -82,7 +83,7 @@ unit(Type) ->
 unit(free, T) -> [dx,dy,dz|T];
 unit(free_2d, T) -> [dx,dy|T];
 unit(intrude, T) -> [{distance,{0.0,9.9E307}}|T];
-unit(_, T) -> [distance|T].
+unit(_, T) -> [distance,skip,skip|T].
 
 flags(free) -> [screen_relative];
 flags(free_2d) -> [screen_relative];
@@ -90,11 +91,10 @@ flags(_) -> [].
 
 move_away_fun(Tv, VsPos0) ->
     fun(view_changed, _Acc) ->
-	    move_away_fun(Tv, VsPos0);
-       (new_falloff, Falloff) ->
-	    VsPos = move_away(Falloff, Tv),
-	    move_away_fun(Tv, VsPos);
-       (_, Acc) -> VsPos0 ++ Acc
+        move_away_fun(Tv, VsPos0);
+       ([_, _, _, Percent|_], A) ->
+        VsPos = move_away(Percent, Tv),
+        VsPos ++ A
     end.
 
 move_away(R0, Tv) ->
@@ -312,13 +312,13 @@ smallest_angle([], _Na, Dot, N) -> {N,Dot}.
 %%
 
 translate_fun(Free) when Free == free; Free == free_2d ->
-    fun(Matrix0, [Dx,Dy,Dz]) ->
+    fun(Matrix0, [Dx,Dy,Dz|_]) ->
 	    #view{azimuth=Az,elevation=El} = wings_view:current(),
 	    M0 = e3d_mat:mul(Matrix0, e3d_mat:rotate(-Az, {0.0,1.0,0.0})),
 	    M1 = e3d_mat:mul(M0, e3d_mat:rotate(-El, {1.0,0.0,0.0})),
 	    {Xt,Yt,Zt} = e3d_mat:mul_point(M1, {Dx,Dy,-Dz}),
 	    e3d_mat:translate(Xt, Yt, Zt);
-       (Matrix0, [Dx,Dy]) ->
+       (Matrix0, [Dx,Dy|_]) ->
 	    #view{azimuth=Az,elevation=El} = wings_view:current(),
 	    M0 = e3d_mat:mul(Matrix0, e3d_mat:rotate(-Az, {0.0,1.0,0.0})),
 	    M1 = e3d_mat:mul(M0, e3d_mat:rotate(-El, {1.0,0.0,0.0})),
@@ -326,7 +326,7 @@ translate_fun(Free) when Free == free; Free == free_2d ->
 	    e3d_mat:translate(Xt, Yt, Zt)
     end;
 translate_fun({Xt0,Yt0,Zt0}) ->
-    fun(_Matrix0, [Dx]) when is_float(Dx) ->
+    fun(_Matrix0, [Dx|_]) when is_float(Dx) ->
 	    Xt = Xt0*Dx,
 	    Yt = Yt0*Dx,
 	    Zt = Zt0*Dx,
@@ -439,9 +439,7 @@ make_tvs(Vs, free, We) ->
 make_tvs(Vs, Vec, _We) -> [{Vec,Vs}].
 
 move_fun(VsPos, ViewMatrix) ->
-    fun(new_falloff, _) ->
-	    move_fun(VsPos, ViewMatrix);
-       (view_changed, NewWe) ->
+    fun(view_changed, NewWe) ->
 	    move_fun(wings_util:update_vpos(VsPos, NewWe), view_matrix());
        ([Dx,Dy,Dz|_], Acc) ->
 	    {Xt,Yt,Zt} = e3d_mat:mul_point(ViewMatrix, {Dx,Dy,-Dz}),
@@ -464,4 +462,3 @@ view_matrix() ->
 
 make_vector(free_2d) -> free_2d;
 make_vector(Vec) -> wings_util:make_vector(Vec).
-    
