@@ -3,12 +3,12 @@
 %%
 %%     This module implements the commands in the selection menu.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_sel_cmd.erl 326 2008-06-07 20:33:23Z antoneos $
+%%     $Id$
 %%
 
 -module(wings_sel_cmd).
@@ -19,8 +19,7 @@
 -export([select_all/1]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldl/3,reverse/1,reverse/2,sort/1,
-		keydelete/3,keymember/3,keysearch/3]).
+-import(lists, [map/2,foldl/3,reverse/1,keymember/3,keysearch/3]).
 
 menu(St) ->
     [{?__(1,"Deselect"),deselect,?__(2,"Clear the selection")},
@@ -28,7 +27,8 @@ menu(St) ->
      {?__(3,"More"),more,more_help(St)},
      {?__(4,"Less"),less,less_help(St)},
      {?__(5,"Similar"),similar,similar_help(St)}]
-    ++ oriented_faces_menu(St) ++ similar_area_faces_menu(St) ++
+    ++ oriented_faces_menu(St) ++ similar_area_faces_menu(St)
+    ++ similar_material_faces(St) ++
     [separator,
      {?__(6,"Edge Loop"),
       {edge_loop,
@@ -154,6 +154,12 @@ similar_area_faces_menu(#st{selmode=face}) ->
 similar_area_faces_menu(_) ->
   [].
 
+similar_material_faces(#st{selmode=face}) ->
+    [{?__(1,"Similar Material"),similar_material,
+      ?__(2,"Select faces with a similar material to those already selected"),[option]}];
+similar_material_faces(_) ->
+    [].
+
 groups_menu(#st{ssels=Ssels}=St) ->
     case gb_trees:is_empty(Ssels) of
 	true -> [];
@@ -261,6 +267,10 @@ command({oriented_faces,Ask}, St) ->
     oriented_faces(Ask, St);
 command({similar_area,Ask}, St) ->
     similar_area(Ask, St);
+	
+command({similar_material,Ask}, St) ->
+    similar_material(Ask, St);
+	
 command({select_group,Id}, St) ->
     {save_state,select_group(Id, St)};
 command({union_group, Id}, St) ->
@@ -337,29 +347,29 @@ by_command({vertices_with,N}, St) ->
     {save_state,wings_sel:make(Sel, vertex, St)};
 by_command({non_quad,all}, St) ->
     Sel = fun(Face, We) ->
-		  wings_face:vertices(Face, We) =/= 4
+		  wings_face:vertices(Face, We) =/= 4 andalso Face >= 0
 	    end,
     {save_state,wings_sel:make(Sel, face, St)};
 by_command({non_quad,odd}, St) ->
     Sel = fun(Face, We) ->
 		  Half = wings_face:vertices(Face, We)/2,
-		  trunc(Half) /= Half
+		  trunc(Half) /= Half andalso Face >= 0
 	    end,
     {save_state,wings_sel:make(Sel, face, St)};
 by_command({non_quad,even}, St) ->
     Sel = fun(Face, We) ->
 		  Half = wings_face:vertices(Face, We)/2,
-		  (Half /= 2) and (trunc(Half) == Half)
+		  (Half /= 2) and (trunc(Half) == Half) andalso Face >= 0
 	    end,
     {save_state,wings_sel:make(Sel, face, St)};
 by_command({faces_with,5}, St) ->
     Sel = fun(Face, We) ->
-		  wings_face:vertices(Face, We) >= 5
+		  wings_face:vertices(Face, We) >= 5 andalso Face >= 0
 	    end,
     {save_state,wings_sel:make(Sel, face, St)};
 by_command({faces_with,N}, St) ->
     Sel = fun(Face, We) ->
-		  N =:= wings_face:vertices(Face, We)
+		  N =:= wings_face:vertices(Face, We) andalso Face >= 0
 	  end,
     {save_state,wings_sel:make(Sel, face, St)};
 by_command(material_edges, St) ->
@@ -632,7 +642,7 @@ similar(#st{selmode=face}=St) ->
     Templates = ordsets:from_list(Seed),
     wings_sel:make(
       fun(Face, WeI) ->
-	      match_templates(make_face_template(Face, WeI), Templates)
+	      Face >= 0 andalso match_templates(make_face_template(Face, WeI), Templates)
       end, face, St);
 similar(#st{selmode=body}=St) ->
     Template0 = wings_sel:fold(fun(_, #we{vp=Vtab,es=Etab,fs=Ftab}, Acc) ->
@@ -782,8 +792,9 @@ material_edges_fun(E, #we{es=Etab}=We) ->
 uv_mapped_faces(St) ->
     wings_sel:make(fun is_uv_mapped_face/2, face, St).
 
-is_uv_mapped_face(Face, We) ->
-    is_uv_mapped_face_1(wings_face:vertex_info(Face, We)).
+is_uv_mapped_face(Face, We) when Face >= 0->
+    is_uv_mapped_face_1(wings_face:vertex_info(Face, We));
+is_uv_mapped_face(_,_) -> false.
 
 is_uv_mapped_face_1([{_,_}|T]) ->
     is_uv_mapped_face_1(T);
@@ -909,7 +920,7 @@ nonplanar_faces(Ask, _St) when is_atom(Ask) ->
 		     fun(Res) -> {select,{by,{nonplanar_faces,Res}}} end);
 nonplanar_faces([Tolerance], St) ->
     Sel = fun(Face, We) ->
-	      not wings_face:is_planar(Tolerance,Face,We)
+	      not wings_face:is_planar(Tolerance,Face,We) andalso Face >= 0
 	  end,
     {save_state,wings_sel:make(Sel, face, St)}.
 
@@ -919,31 +930,179 @@ nonplanar_faces([Tolerance], St) ->
 
 oriented_faces(_, #st{selmode=Mode}) when Mode =/= face ->
     keep;					%Wrong mode (invoked through hotkey).
+
 oriented_faces(Ask, _St) when is_atom(Ask) ->
-    Qs = [{label,?__(1,"Angle tolerance")},
-	  {text,1.0E-3,[{range,{0.0,180.0}}]}],
+    Connected = wings_pref:get_value(similar_normals_connected,false),
+    {Save,Angle} = case wings_pref:get_value(similar_normals_angle,{false,1.0E-3}) of
+        {true,A} -> {true,A};
+        {false,_} -> {false,1.0E-3}
+    end,
+    Qs = [{vframe,
+           [{hframe,[{slider,{text,Angle,[{range,{1.0E-3,180.0}}]}}]}],
+           [{title,?__(1,"Angle Tolerance")}]},
+          {?__(3,"Connected Faces Only"),Connected},
+          {?__(5,"Save Angle"),Save}],
     wings_ask:dialog(Ask, ?__(2,"Select Similarly Oriented Faces"),
-		     [{hframe,Qs}],
-		     fun(Res) -> {select,{oriented_faces,Res}} end);
-oriented_faces([Tolerance], St) ->
+    [{vframe,Qs}],
+    fun(Res) -> {select,{oriented_faces,Res}} end);
+
+oriented_faces([Tolerance,Connected,Save], #st{selmode=face, sel=[]}) ->
+    wings_pref:set_value(similar_normals_connected,Connected),
+    wings_pref:set_value(similar_normals_angle,{Save,Tolerance}),
+    wings_u:error(?__(4,"At least one face must be selected"));
+
+oriented_faces([Tolerance,false,Save], St) ->
+    wings_pref:set_value(similar_normals_connected,false),
+    wings_pref:set_value(similar_normals_angle,{Save,Tolerance}),
     CosTolerance = math:cos(Tolerance * (math:pi() / 180.0)),
-    Normals = wings_sel:fold(fun(Sel0, We, A) ->
-				 [wings_face:normal(SelI, We) ||
-				     SelI <- gb_sets:to_list(Sel0)] ++ A
-			     end, [], St),
+    Normals0 = wings_sel:fold(fun(Faces, We, A) ->
+                [wings_face:normal(F, We) ||
+                    F <- gb_sets:to_list(Faces)] ++ A
+              end, [], St),
+	Normals = lists:usort(Normals0),
     Sel = fun(Face, We) ->
-	      Normal = wings_face:normal(Face,We),
-	      any_matching_normal(CosTolerance, Normal, Normals)
-	  end,
-    {save_state,wings_sel:make(Sel, face, St)}.
+          Normal = wings_face:normal(Face,We),
+          any_matching_normal(CosTolerance, Normal, Normals)
+      end,
+    {save_state,wings_sel:make(Sel, face, St)};
+
+oriented_faces([Tolerance,true,Save], St0) ->
+    wings_pref:set_value(similar_normals_angle,{Save,Tolerance}),
+    wings_pref:set_value(similar_normals_connected,true),
+    CosTolerance = math:cos(Tolerance * (math:pi() / 180.0)),
+    Sel = wings_sel:fold(fun(Faces, #we{id=Id}=We, A) ->
+	        Normals0 = foldl(fun(F,Acc) ->
+			        [wings_face:normal(F, We)|Acc]
+		    end,[],gb_sets:to_list(Faces)),
+            Normals = lists:usort(Normals0),
+            [{Id,norm_search(Faces,Normals,CosTolerance,We,Faces)}|A]
+    end, [], St0),
+    wings_sel:set(face,Sel,St0).
+
+norm_search(Faces,Normals,CosTolerance,We,LastSel) ->
+    Fs0 = wings_face:extend_border(LastSel, We),
+    Fs1 = gb_sets:subtract(Fs0,Faces),
+    AddSel = check_face_normals(Fs1,Normals,CosTolerance,We,gb_sets:empty()),
+    case gb_sets:is_empty(AddSel) of
+        true -> Faces;
+        false ->
+            Faces1 = gb_sets:union(AddSel,Faces),
+            norm_search(Faces1,Normals,CosTolerance,We,AddSel)
+    end.
+
+check_face_normals(Fs0,Normals,CosTolerance,We,Selection)->
+    case gb_sets:is_empty(Fs0) of
+      true -> Selection;
+      false ->
+        {F,Fs1} = gb_sets:take_smallest(Fs0),
+        Normal = wings_face:normal(F, We),
+        Sel = case any_matching_normal(CosTolerance, Normal, Normals) of
+          true -> gb_sets:add(F,Selection);
+          false -> Selection
+        end,
+        check_face_normals(Fs1,Normals,CosTolerance,We,Sel)
+    end.
 
 any_matching_normal(_,_,[]) ->
     false;
-any_matching_normal(CosTolerance, Normal, [N|T]) ->
-    Dot = e3d_vec:dot(N, Normal),
+any_matching_normal(CosTolerance, Norm, [N|T]) ->
+    Dot = e3d_vec:dot(N, Norm),
     if
       Dot >= CosTolerance -> true;
-      true -> any_matching_normal(CosTolerance, Normal, T)
+      true ->  any_matching_normal(CosTolerance, Norm, T)
+    end.
+
+%%%
+%%% Select faces of the same material.
+%%%
+similar_material(_, #st{selmode=Mode}) when Mode =/= face ->
+    keep;					%Wrong mode (invoked through hotkey).
+
+similar_material(Ask, _St) when is_atom(Ask) ->
+    Connected = wings_pref:get_value(similar_materials_connected,false),
+    Qs = [{?__(2,"Connected Faces Only"),Connected}],
+    wings_ask:dialog(Ask, ?__(4,"Select Faces with the same Material"),
+    [{vframe,Qs}],
+    fun(Res) -> {select,{similar_material,Res}} end);
+
+similar_material([Connected], #st{selmode=face, sel=[]}) ->
+    wings_pref:set_value(similar_materials_connected,Connected),
+    wings_u:error(?__(3,"At least one face must be selected"));
+
+similar_material([false], St) ->
+    Materials = wings_sel:fold(fun
+       (Faces, #we{mode=vertex}=We, A) ->
+        foldl(fun(F,Acc) ->
+               [wings_color:average([C || [_|C]
+                 <- wings_face:vinfo_ccw(F, We)])|Acc]
+        end,A,gb_sets:to_list(Faces));
+       (Faces, We, A) ->
+               [wings_facemat:face(SelI, We) ||
+                   SelI <- gb_sets:to_list(Faces)] ++ A
+             end, [], St),
+    Sel = fun	
+        (F, #we{mode=vertex}=We) ->
+          Col = wings_color:average([C||[_|C] <- wings_face:vinfo_ccw(F, We)]),
+          any_matching_material(Col,Materials);
+        (Face, We) ->
+          Mat = wings_facemat:face(Face,We),
+          any_matching_material(Mat, Materials)
+      end,
+    wings_pref:set_value(similar_materials_connected,false),
+    {save_state,wings_sel:make(Sel, face, St)};
+
+similar_material([true], St0) ->
+    Selection = wings_sel:fold(fun
+        (Faces, #we{id=Id,mode=vertex}=We, A) ->
+            AllCols = foldl(fun(F,Acc) ->
+                Cols = [C || [_|C] <- wings_face:vinfo_ccw(F, We)],
+                [wings_color:average(Cols)|Acc]
+            end,A,gb_sets:to_list(Faces)),
+            Colours = lists:usort(AllCols),
+            [{Id,mat_search(Faces,Colours,We,Faces)}|A];
+        (Faces, #we{id=Id}=We, A) ->
+            AllMats = [wings_facemat:face(Face, We) || Face <- gb_sets:to_list(Faces)],
+            Materials = lists:usort(AllMats),
+            [{Id,mat_search(Faces,Materials,We,Faces)}|A]
+    end, [], St0),
+    wings_sel:set(face,Selection,St0).
+
+mat_search(Faces,Colours,We,LastSel) ->
+    Fs0 = wings_face:extend_border(LastSel, We),
+    Fs1 = gb_sets:subtract(Fs0,Faces),
+    AddSel = check_face_colours(Fs1,Colours,We,gb_sets:empty()),
+    case gb_sets:is_empty(AddSel) of
+        true -> Faces;
+        false ->
+            Faces1 = gb_sets:union(AddSel,Faces),
+            mat_search(Faces1,Colours,We,AddSel)
+    end.
+
+check_face_colours(Fs0,Colours,We,Selection)->
+    case gb_sets:is_empty(Fs0) of
+      true -> Selection;
+      false ->
+        {F,Fs1} = gb_sets:take_smallest(Fs0),
+        Colour = face_info(F,We),
+        Sel = case any_matching_material(Colour,Colours) of
+          true -> gb_sets:add(F,Selection);
+          false -> Selection
+        end,
+        check_face_colours(Fs1,Colours,We,Sel)
+    end.
+
+face_info(F,#we{mode=vertex}=We) ->
+    Cols = [C || [_|C] <- wings_face:vinfo_ccw(F, We)],
+    wings_color:average(Cols);
+face_info(F,We) ->
+    wings_facemat:face(F, We).
+
+any_matching_material(_,[]) ->
+    false;
+any_matching_material(Material, [Mat|T]) ->
+    if
+      Material == Mat -> true;
+      true -> any_matching_material(Material, T)
     end.
 
 %%%
@@ -1197,7 +1356,9 @@ similar_area([Tolerance], St) ->
     St2 = wings_sel:make(SelFun, face, St),
     {save_state,St2}.
 
-is_area_similar(Area1, Tolerance, Face, We) ->
+is_area_similar(Area1, Tolerance, Face, We) when Face >= 0 ->
     Area2 = wings_face:area(Face, We),
     AreaDiff = abs(Area1-Area2),
-    AreaDiff =< Tolerance.
+    AreaDiff =< Tolerance;
+is_area_similar(_,_,_,_) ->
+    false.
