@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_toolbar.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_toolbar).
@@ -47,9 +47,6 @@ init_button() ->
 get_button_event(But) ->
     {replace,fun(Ev) -> button_event(Ev, But) end}.
 
-button_event(got_focus, _) ->
-    wings_wm:dirty(),
-    keep;
 button_event({window_updated,_}, But) ->
     get_button_event(button_resized(But));
 button_event(redraw, #but{buttons=undefined}=But0) ->
@@ -78,6 +75,8 @@ button_event({current_state,#st{selmode=Mode,sh=Sh}}, But) ->
     get_button_event(But#but{mode=Mode,sh=Sh});
 button_event({mode_restriction,Restr}, #but{restr=Restr}) ->
     keep;
+button_event({mode_restriction,Restr}, #but{all_buttons=undefined}=But) ->
+    get_button_event(But#but{restr=Restr});
 button_event({mode_restriction,Restr}, #but{all_buttons=AllButtons}=But) ->
     Buttons = button_restrict(AllButtons, Restr),
     wings_wm:dirty(),
