@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_u.erl 207 2008-01-04 16:18:30Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_u).
@@ -22,10 +22,14 @@
 -include("wings.hrl").
 -import(lists, [member/2,foreach/2]).
 
+-spec error([any()]) -> no_return().
+
 error(Message) when is_list(Message) ->
     wings_pb:cancel(),
     throw({command_error,Message}).
 
+-spec error([any()], [any()]) -> no_return().
+
 error(Format, Arg) ->
     error(lists:flatten(io_lib:format(Format, Arg))).
 
@@ -121,23 +125,35 @@ crash_log(WinName, Reason, StackTrace) ->
     LogName.
 
 caption(#st{file=undefined}=St) ->
-    Caption = wings_util:wings(),
+    Caption = wings(),
     sdl_video:wm_setCaption(Caption, Caption),
     St;
 caption(#st{saved=true,file=Name}=St) ->
-    Caption = wings_util:wings() ++ " - " ++ filename:basename(Name),
+    Caption = wings() ++ " - " ++ filename:basename(Name),
     sdl_video:wm_setCaption(Caption, Caption),
     St;
 caption(#st{saved=auto,file=Name}=St) ->
-    Caption = wings_util:wings() ++ " - " ++ filename:basename(Name) ++
+    Caption = wings() ++ " - " ++ filename:basename(Name) ++
 	"* [" ++ ?__(1,"auto-saved") ++ "]",
     sdl_video:wm_setCaption(Caption, Caption),
     St;
 caption(#st{file=Name}=St) ->
-    Caption = wings_util:wings() ++ " - " ++ filename:basename(Name) ++ "*",
+    Caption = wings() ++ " - " ++ filename:basename(Name) ++ "*",
     sdl_video:wm_setCaption(Caption, Caption),
     St.
 
+wings() ->
+    case ?wings_branch of
+	"" -> debug("Wings3D");
+	_ -> "Wings3D " ++ ?wings_version ++ " (" ++ ?wings_branch ++ ")"
+    end.
+
+-ifdef(DEBUG).
+debug(Caption) -> Caption ++ " [debug]".
+-else.
+debug(Caption) -> Caption.
+-endif.
+
 %%%
 %%% Local functions.
 %%%
