@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_vec.erl 293 2008-03-21 09:57:09Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_vec).
@@ -19,7 +19,7 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 
--import(lists, [foldl/3,keydelete/3,reverse/1,member/2,last/1]).
+-import(lists, [reverse/1,member/2,last/1]).
 
 -record(ss, {f,					%Fun.
 	     selmodes,				%Legal selection modes.
@@ -209,8 +209,51 @@ handle_event_4({action,{select,Cmd}}, Ss, St0) ->
 	{save_state,St} -> filter_sel_command(Ss, St);
 	keep -> keep
     end;
+
 handle_event_4({action,{view,auto_rotate}}, _, _) ->
     keep;
+
+%%%% Vector op Highlight Aim
+handle_event_4({action,{view,highlight_aim}}, Ss, St0) ->
+    case wings_pref:get_value(use_temp_sel) of
+      false ->
+	    handle_event_4({action,{view,aim}}, Ss, St0);
+      true ->
+        HL0 = wings_pref:get_value(highlight_aim_at_unselected),
+        HL1 = wings_pref:get_value(highlight_aim_at_selected),
+        {_,X,Y} = wings_wm:local_mouse_state(),
+        {{_,Cmd},St1} = case wings_pick:do_pick(X, Y, St0) of
+          {add,_,St} when HL0 =:= true ->
+              {{view,highlight_aim},{add,St0,St}};
+          {delete,_,St} when HL1 =:= true ->
+              {{view,highlight_aim},{delete,St0,St}};
+          _Other -> 
+              {{view,aim}, St0}
+        end,
+        St2 = wings_view:command(Cmd,St1),
+        get_event(Ss, St2)
+    end;
+
+%%%% Temporary Select Aim for View commands with no selection
+handle_event_4({action,{view,Cmd}}, Ss, #st{sel=[]}=St0)
+  when Cmd == align_to_selection; Cmd == aim; Cmd == frame ->
+    case wings_pref:get_value(use_temp_sel) of
+    false ->
+      St = wings_view:command(Cmd, St0),
+      get_event(Ss, St);
+    true ->
+      {_,X,Y} = wings_wm:local_mouse_state(),
+      case wings_pick:do_pick(X, Y, St0) of
+        {add,_,St} ->
+          St1 = wings_view:command(Cmd,St),
+		  St2 = wings_sel:clear(St1),
+          get_event(Ss, St2);
+        _Other ->
+          St = wings_view:command(Cmd, St0),
+          get_event(Ss, St)
+      end
+    end;
+
 handle_event_4({action,{view,Cmd}}, Ss, St0) ->
     St = wings_view:command(Cmd, St0),
     get_event(Ss, St);
@@ -255,9 +298,17 @@ pick_next_1([], Res, #ss{cb=Cb,new_st=NewSt}, _) ->
     wings:clear_mode_restriction(),
     wings_wm:later(build_result(Res, Cb, NewSt)),
     pop;
-pick_next_1([{Fun0,Desc}|More], _Done, Ss, St) when is_function(Fun0) ->
+pick_next_1([{Fun0,Desc}|More], Done, Ss, St) when is_function(Fun0) ->
     Fun = fun(message, Right) -> common_message(Desc, More, no, Right);
-	     (A, B) -> Fun0(A, B)
+	     (A, B) ->
+		     case Fun0(A, B) of
+		       {result,Res} ->
+		           %% the 'result' tag allows plugins to add custom secondary
+		           %% selections to the Done accumulator.
+		           %% See example in wpc_circularise.
+		           {More,[Res|Done]};
+		       Other -> Other
+		     end
 	  end,
     get_event(Ss#ss{f=Fun,is_axis=false,vec=none,info=""}, wings_sel:reset(St));
 pick_next_1([{Type,Desc}|More], Done, Ss, St0) ->
@@ -428,6 +479,15 @@ check_vector(#st{selmode=vertex,sel=[{Id0,Sel0},{Id1,Sel1}],shapes=Shs}=St) ->
         Str = guard_string(),
         {none,Str}
     end;
+check_vector(#st{selmode=edge,sel=[{Id0,Sel0},{Id1,Sel1}],shapes=Shs}) ->
+        We0 = gb_trees:get(Id0,Shs),
+        We1 = gb_trees:get(Id1,Shs),
+        EL0 = gb_sets:to_list(Sel0),
+        EL1 = gb_sets:to_list(Sel1),
+        case length(EL0) + length(EL1) of
+          2 -> get_vec(edge, EL0 ++ EL1, [We0, We1]);
+          _ -> get_vec(edge, [{EL0}, {EL1}], [We0, We1])
+        end;
 
 check_vector(#st{selmode=Mode,sel=[{Id0,Sel0},{Id1,Sel1}],shapes=Shs}) ->
     SelSize = gb_sets:size(Sel0) + gb_sets:size(Sel1),
@@ -453,8 +513,9 @@ check_vector(_) ->
     {none,Str}.
 
 guard_string() ->
-    ?__(1,"Select parts of one object only")++
-    ?__(2,", or select one element in each of two objects")++
+    ?__(1,"Select parts of one object only") ++
+    ?__(2,", or select one element in each of two objects") ++
+	?__(4,",\nor a single edge loop on each object ") ++
     ?__(3,", or any three vertices.").
 
 get_pos(Vs,We,Acc) ->
@@ -492,6 +553,21 @@ get_vec(vertex, [Va, Vb], [We0, We1]) ->
       {?__(11,"Average of vertex normals saved as axis."),
        ?__(12,"Use direction between vertices as axis")}}];
 
+%% Use two edge loops on separate objects.
+get_vec(edge, [{Edges1},{Edges2}], [We1,We2]) ->
+    Loop1 = wings_edge_loop:edge_loop_vertices(Edges1, We1),
+    Loop2 = wings_edge_loop:edge_loop_vertices(Edges2, We2),
+    case {Loop1, Loop2} of
+    {[Vs1],[Vs2]} ->
+        LoopCenter1 = wings_vertex:center(Vs1, We1),
+        LoopCenter2 = wings_vertex:center(Vs2, We2),
+        Center = e3d_vec:average([LoopCenter1, LoopCenter2]),
+        Vec = e3d_vec:norm_sub(LoopCenter1, LoopCenter2),
+        [{{Center,Vec},?__(28,"Axis between edge loop centers saved as axis.")}];		
+    _Other ->
+        [{none,guard_string()}]
+    end;
+
 get_vec(edge, [Edge1,Edge2], [#we{es=Etab0,vp=Vtab0},#we{es=Etab1,vp=Vtab1}]) ->
     #edge{vs=Va1,ve=Vb1} = gb_trees:get(Edge1, Etab0),
     #edge{vs=Va2,ve=Vb2} = gb_trees:get(Edge2, Etab1),
@@ -513,6 +589,7 @@ get_vec(edge, [Edge1,Edge2], [#we{es=Etab0,vp=Vtab0},#we{es=Etab1,vp=Vtab1}]) ->
       {?__(25,"Cross product of edges saved as axis"),
        ?__(26,"Save direction between edges")}}];
 
+
 get_vec(face, [Face1, Face2], [We0, We1]) ->
     Center1 = wings_face:center(Face1, We0),
     Center2 = wings_face:center(Face2, We1),
@@ -568,8 +645,14 @@ get_vec(edge, Edges, #we{vp=Vtab}=We) ->
 	    Center = wings_vertex:center(Vs, We),
 	    Vec = wings_face:face_normal_ccw(Vs, Vtab),
 	    [{{Center,Vec},?__(6,"Edge loop normal saved as axis.")}];
+	[Vs1,Vs2] ->
+	    LoopCenter1 = wings_vertex:center(Vs1, We),
+	    LoopCenter2 = wings_vertex:center(Vs2, We),
+		Center = e3d_vec:average([LoopCenter1, LoopCenter2]),
+	    Vec = e3d_vec:norm_sub(LoopCenter1, LoopCenter2),
+	    [{{Center,Vec},?__(28,"Axis between edge loop centers saved as axis.")}];		
 	_Other ->
-	    [{none,?__(7,"Multi-edge selection must form a single closed edge loop.")}]
+	    [{none,?__(29,"Multi-edge selection must form either a single or two closed edge loops.")}]
     end;
 
 %% Vertex normal
