@@ -3,12 +3,12 @@
 %%
 %%     This module implements most of the commands in the View menu.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_view.erl 296 2008-03-24 17:23:50Z optigon $
+%%     $Id$
 %%
 
 -module(wings_view).
@@ -17,7 +17,7 @@
 	 init/0,initial_properties/0,delete_all/1,
 	 current/0,set_current/1,
 	 load_matrices/1,projection/0,
-	 modelview/0,modelview/1,
+	 modelview/0,modelview/1,align_view_to_normal/1,
 	 eye_point/0,export_views/1,import_views/2,camera_info/2,
 	 freeze_mirror/1]).
 
@@ -25,83 +25,82 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
--import(lists, [foreach/2,foldl/3,zip/2]).
+-import(erlang, [max/2]).
+-import(lists, [foldl/3,zip/2]).
 
 menu(#st{views={CurrentView,Views}}=St) ->
     L = wings_pref:get_value(number_of_lights),
-    [{?__(1,"Ground Plane"),show_groundplane,?__(2,"Show the ground plane"),
-      crossmark(show_groundplane)},
-     {?__(3,"Axes"),show_axes,?__(4,"Show the coordinate axes"),crossmark(show_axes)},
-     separator,
-     {?__(5,"Workmode"),workmode,?__(6,"Toggle flat/smooth shading"),
-      crossmark(workmode)},
-     {?__(61,"Clipping Planes"),clip_plane,?__(62,"Toggle extra clipping planes (use default axis)"),
-      crossmark(clip_plane)},
+    [{?__(68,"Show"),{show,
+     [{?__(1,"Ground Plane"),show_groundplane,?__(2,"Show the ground plane"),
+       crossmark(show_groundplane)},
+      {?__(3,"Axes"),show_axes,?__(4,"Show the coordinate axes"),crossmark(show_axes)},
+      {?__(48,"Show Info Text"),show_info_text,
+       ?__(49,"Show an informational text at the top of this Geometry window"),
+       crossmark(show_info_text)},
+        separator,
+      {?__(17,"Show Saved BB"),show_bb,?__(18,"Display any saved bounding box"),crossmark(show_bb)},
+      {?__(61,"Clipping Planes"),clip_plane,?__(62,"Toggle extra clipping planes (use default axis)"),
+       crossmark(clip_plane)},
+      {?__(23,"Show Normals"),show_normals,?__(24,"Show normals for selected elements"),
+       crossmark(show_normals)},
+        separator,
+      {?__(42,"Show Colors"),show_colors,
+       ?__(43,"Show vertex colors on objects in \"vertex\" mode"),
+       crossmark(show_colors)},
+      {?__(44,"Show Materials"),show_materials,
+       ?__(45,"Show materials on objects in \"material\" mode"),
+       crossmark(show_materials)},
+      {?__(46,"Show Textures"),show_textures,
+       ?__(47,"Show the texture on objects in \"material\" mode"),
+       crossmark(show_textures)}]}},
      separator,
      {?__(7,"Wireframe"),wireframe,?__(8,"Display selected objects as a wireframe (same for all objects if nothing is selected)")},
      {?__(9,"Shade"),shade,?__(10,"Display selected objects as shaded (same for all objects if nothing is selected)")},
      {?__(11,"Toggle Wireframe"),toggle_wireframe,
       ?__(12,"Toggle display mode for selected objects (same for all objects if nothing is selected)"),wireframe_crossmark(St)},
+     {?__(19,"Show Edges"),show_edges,?__(20,"Show edges in workmode"),crossmark(show_edges)},
+     {?__(21,"Show Wireframe Backfaces"),show_wire_backfaces,
+      ?__(22,"Show wireframe backfaces"),crossmark(show_wire_backfaces)},
      separator,
+     {?__(5,"Workmode"),workmode,?__(6,"Toggle flat/smooth shading"),
+      crossmark(workmode)},
      {?__(13,"Toggle Proxy Mode"),smooth_proxy,
       ?__(14,"Toggle the smooth proxy mode for selected objects")},
      {?__(15,"Quick Smoothed Preview"),quick_preview,
       ?__(16,"Toggle the smooth proxy mode for all objects")},
-     separator,
-     {?__(17,"Show Saved BB"),show_bb,?__(18,"Display any saved bounding box"),crossmark(show_bb)},
-     {?__(19,"Show Edges"),show_edges,?__(20,"Show edges in workmode"),crossmark(show_edges)},
-     {?__(21,"Show Wireframe Backfaces"),show_wire_backfaces,
-      ?__(22,"Show wireframe backfaces"),crossmark(show_wire_backfaces)},
-     {?__(23,"Show Normals"),show_normals,?__(24,"Show normals for selected elements"),
-      crossmark(show_normals)},
-     separator,
-     {?__(25,"Reset View"),reset,?__(26,"Reset view to the default position")},
-     {?__(27,"Aim"),aim,?__(28,"Aim the camera at the selected element")},
-     {?__(29,"Frame"),frame,?__(30,"Dolly to show all selected elements (or all objects if nothing is selected)")},
-	 {?__(65,"Frame Disregards Mirror"),frame_mode,crossmark(frame_disregards_mirror)},
-     {?__(31,"Orthographic View"),orthogonal_view,
-      ?__(32,"Toggle between orthographic and perspective views"),
-      crossmark(orthogonal_view)},
-     {?__(33,"Saved Views: ")++integer_to_list(tuple_size(Views)),
-      {views,views_submenu(CurrentView, Views)}},
-     separator,
-     {?__(34,"Camera Settings..."),camera_settings,?__(35,"Set field of view, and near and far clipping planes")},
-     separator,
-     {?__(36,"Scene Lights"),scene_lights,
-      ?__(37,"Use the lights defined in the scene"),
-      crossmark(scene_lights)},
-     {one_of(L == 1, ?__(38,"Two Lights"),?__(39,"One Light")),toggle_lights,
-      one_of(L == 1, ?__(40,"Use two work lights"),
-	     ?__(41,"Use one work light"))},
-%% If there should be shaders, there should be a better mechanism for choosing shaders,
-%% and preferably a way to turn them of since they have limited use in a modeller.
-%%    -- Bjorn G
-%%     {?__(63,"Activate Next Shader"),cycle_shaders,
-%%      ?__(64,"Cycle through all vertex/fragment shaders")},
-     separator,
-     {?__(42,"Show Colors"),show_colors,
-      ?__(43,"Show vertex colors on objects in \"vertex\" mode"),
-      crossmark(show_colors)},
-     {?__(44,"Show Materials"),show_materials,
-      ?__(45,"Show materials on objects in \"material\" mode"),
-      crossmark(show_materials)},
-     {?__(46,"Show Textures"),show_textures,
-      ?__(47,"Show the texture on objects in \"material\" mode"),
-      crossmark(show_textures)},
-     {?__(48,"Show Info Text"),show_info_text,
-      ?__(49,"Show an informational text at the top of this Geometry window"),
-      crossmark(show_info_text)},
-     separator,
-     {?__(50,"View Along"),{along,[{?__(51,"+X"),x},
-			   {?__(52,"+Y"),y},
-			   {?__(53,"+Z"),z},
-			   {?__(54,"-X"),neg_x},
-			   {?__(55,"-Y"),neg_y},
-			   {?__(56,"-Z"),neg_z}]}},
-     separator,
-     {?__(57,"Align to Selection"),align_to_selection,
-      ?__(58,"Align the view to the normal of the selection")},
-     {?__(59,"Auto Rotate"),auto_rotate,?__(60,"Spin the view")}].
+     separator |
+     shader_submenu() ++
+     [{?__(36,"Scene Lights"),scene_lights,
+       ?__(37,"Use the lights defined in the scene"),
+       crossmark(scene_lights)},
+      {one_of(L == 1, ?__(38,"Two Lights"),?__(39,"One Light")),toggle_lights,
+       one_of(L == 1, ?__(40,"Use two work lights"),
+       ?__(41,"Use one work light"))},
+      separator,
+      {?__(31,"Orthographic View"),orthogonal_view,
+       ?__(32,"Toggle between orthographic and perspective views"),
+       crossmark(orthogonal_view)},
+      {?__(25,"Reset View"),reset,?__(26,"Reset view to the default position")},
+      {?__(27,"Aim"),aim,?__(28,"Aim the camera at the selected element")},
+      {?__(66,"Highlight Aim"),highlight_aim,
+       ?__(67,"Aim camera at mouseover highlight. (Requires 'Use Highlight as Temporary Selection' enabled, and an assigned hotkey)")},
+      {?__(29,"Frame"),frame,?__(30,"Dolly to show all selected elements (or all objects if nothing is selected)")},
+      {?__(65,"Frame Disregards Mirror"),frame_mode,crossmark(frame_disregards_mirror)},
+      {?__(57,"Align to Selection"),align_to_selection,
+       ?__(58,"Align the view to the normal of the selection")},
+      separator,
+      {?__(33,"Saved Views: ")++integer_to_list(tuple_size(Views)),
+       {views,views_submenu(CurrentView, Views)}},
+      {?__(50,"View Along"),{along,[{?__(51,"+X"),x},
+				    {?__(52,"+Y"),y},
+				    {?__(53,"+Z"),z},
+				    {?__(54,"-X"),neg_x},
+				    {?__(55,"-Y"),neg_y},
+				    {?__(56,"-Z"),neg_z}]}},
+      separator,
+      {?__(34,"Camera Settings..."),camera_settings,?__(35,"Set field of view, and near and far clipping planes")},
+      separator,
+      {?__(59,"Auto Rotate"),auto_rotate,?__(60,"Spin the view")}]].
 
 crossmark(Key) ->
     Val = case wings_pref:get_value(Key) of
@@ -136,7 +135,7 @@ wireframe_crossmark(#st{sel=Sel0}) ->
 
 views_submenu(CurrentView, Views) ->
     {_,_,_,H} = wings_wm:viewport(desktop),
-    Lines = wings_util:max((H div ?LINE_HEIGHT) - 3, 4),
+    Lines = max((H div ?LINE_HEIGHT) - 3, 4),
     S = tuple_size(Views),
     C = if S > 0 -> view_index(CurrentView, S); true -> 0 end,
     [{?__(1,"Next"),next,views_submenu_help(CurrentView, Views, next)},
@@ -297,28 +296,28 @@ command(quick_preview, St) ->
 command(orthogonal_view, St) ->
     toggle_option(orthogonal_view),
     St;
-command(show_textures, St) ->
+command({show,show_textures}, St) ->
     toggle_option(show_textures),
     wings_dl:map(fun(#dlo{src_we=#we{mode=material}}=D, _) ->
 			 D#dlo{work=none,smooth=none,proxy_faces=none};
 		    (D, _) -> D
 		 end, []),
     St;
-command(show_materials, St) ->
+command({show,show_materials}, St) ->
     toggle_option(show_materials),
     wings_dl:map(fun(#dlo{src_we=#we{mode=material}}=D, _) ->
 			 D#dlo{work=none,smooth=none,proxy_faces=none};
 		    (D, _) -> D
 		 end, []),
     St;
-command(show_colors, St) ->
+command({show,show_colors}, St) ->
     toggle_option(show_colors),
     wings_dl:map(fun(#dlo{src_we=#we{mode=vertex}}=D, _) ->
 			 D#dlo{work=none,smooth=none,proxy_faces=none};
 		    (D, _) -> D
 		 end, []),
     St;
-command(show_normals, St) ->
+command({show,show_normals}, St) ->
     Bool = wings_pref:get_value(show_normals),
     wings_pref:set_value(show_normals, not Bool),
     case Bool of
@@ -336,6 +335,15 @@ command(show_edges, St) ->
 	    wings_dl:map(fun(D, _) -> D#dlo{hard=none} end, []),
 	    St
     end;
+command(highlight_aim, {add,St0,St}) ->
+    highlight_aim(St0,St),
+    St0;
+command(highlight_aim, {delete,St0,St}) ->
+    highlight_aim(St,St0),
+    St0;
+command(highlight_aim, St) ->
+    aim(St),
+    St;
 command(aim, St) ->
     aim(St),
     St;
@@ -364,9 +372,9 @@ command(align_to_selection, St) ->
 command(toggle_lights, St) ->
     toggle_lights(),
     St;
-%% command(cycle_shaders, St) ->
-%%     cycle_shaders(),
-%%     St;
+command({shader_set,N}, St) ->
+    shader_set(N),
+    St;
 command(camera_settings, St) ->
     camera(),
     St;
@@ -816,6 +824,9 @@ get_event(Tim) ->
 %%% Other stuff.
 %%%
 
+toggle_option({show,Key}) ->
+%% process Show menu items
+    toggle_option(Key);
 toggle_option(Key) ->
     case wings_wm:lookup_prop(Key) of
 	none ->
@@ -841,7 +852,6 @@ init() ->
     wings_pref:set_default(show_materials, true),
     wings_pref:set_default(show_textures, true),
     wings_pref:set_default(frame_disregards_mirror, false),
-
     wings_pref:set_default(scene_lights, false).
 
 initial_properties() ->
@@ -935,6 +945,54 @@ view_matrix(#view{origin=Origin,distance=Dist,azimuth=Az,elevation=El,
     M2 = e3d_mat:mul(M1, e3d_mat:rotate(-El, {1.0,0.0,0.0})),
     e3d_mat:mul(M2, e3d_mat:translate(-PanX, -PanY, Dist)).
 
+%%%% Highlight Aim is called through the View|Aim command (see wings:do_hotkey/2)
+highlight_aim(St0, #st{selmode=body}=St) ->
+    highlight_aim(wings_sel_conv:mode(vertex,St0),wings_sel_conv:mode(vertex,St));
+
+highlight_aim(#st{sel=Sel0}, #st{selmode=Mode, sel=Sel, shapes=Shs}) ->
+    Elems0 = highlight_aim_1(Sel0),
+    Elems = highlight_aim_1(Sel),
+    {TargetId,Target} = highlight_target(Elems0, Elems),
+    We = gb_trees:get(TargetId, Shs),
+    TargetVs = target_vs(Mode, Target, We),
+    Origin0 = wings_vertex:center(TargetVs,We),
+    Origin = e3d_vec:neg(Origin0),
+    #view{distance=Dist0} = View = current(),
+    Dist = case e3d_vec:dist(eye_point(), Origin0) of
+           D when D < Dist0 -> D;
+           _Other -> Dist0
+       end,
+    set_current(View#view{origin=Origin,distance=Dist,pan_x=0.0,pan_y=0.0}).
+
+highlight_aim_1(Sel) ->
+    lists:foldl(fun({Id,Elements}, Acc) ->
+        [{Id, gb_sets:to_list(Elements)}|Acc]
+        end, [], Sel).
+
+highlight_target(Elems0, Elems) ->
+    case length(Elems0) == length(Elems) of
+      true -> 
+        highlight_aim_2(Elems0, Elems);
+      false -> 
+        [TargetElems] = Elems -- Elems0,
+        TargetElems
+    end.
+
+highlight_aim_2([Items0|Elems0],[Items|Elems]) when Items0 == Items ->
+    highlight_aim_2(Elems0,Elems);
+
+highlight_aim_2([{_,Items0}|_Elems0],[{Id,Items}|_Elems]) ->
+    Target = Items -- Items0,
+    {Id,Target}.
+
+target_vs(vertex, Target, _) ->
+    Target;
+target_vs(edge, Target, We) ->
+    wings_edge:to_vertices(Target,We);
+target_vs(face, Target, We) ->
+    wings_face:to_vertices(Target,We).
+
+%%%% Aim Camera
 aim(#st{sel=[]}) ->
     View = current(),
     set_current(View#view{origin=e3d_vec:zero()});
@@ -1110,20 +1168,54 @@ toggle_lights() ->
 	     end,
     wings_pref:set_value(number_of_lights, Lights).
 
-%% cycle_shaders() ->
-%%     case wings_gl:support_shaders() of
-%% 	true ->
-%% 	    NumShaders = wings_pref:get_value(number_of_shaders),
-%% 	    NumProgs = tuple_size(get(light_shaders)),
-%% 	    Shaders = case NumShaders of
-%% 			  NumProgs -> 1;
-%% 			  _ -> NumShaders+1
-%% 		      end,
-%% 	    wings_pref:set_value(number_of_lights, 2),
-%% 	    wings_pref:set_value(number_of_shaders, Shaders);
-%% 	false ->
-%% 	    toggle_lights()
-%%     end.
+shader_set(N) ->
+    case wings_gl:support_shaders() of
+	true ->
+	    NumShaders = wings_pref:get_value(number_of_shaders),
+	    NumProgs = tuple_size(get(light_shaders)),
+	    case N of
+		next -> Shaders = case NumShaders of
+				       NumProgs -> 1;
+				       _ -> NumShaders+1
+				  end;
+		prev -> Shaders = case NumShaders of
+				       1 -> NumProgs;
+				       _ -> NumShaders-1
+				  end;
+		_ -> Shaders = N
+	    end,
+	    wings_pref:set_value(number_of_lights, 2),
+	    wings_pref:set_value(number_of_shaders, Shaders);
+	false ->
+	    toggle_lights()
+    end.
+
+shader_submenu() ->
+    case get(light_shaders) of
+	undefined ->
+	    [];
+	Progs0 when is_tuple(Progs0) ->
+	    Progs = tuple_to_list(Progs0),
+	    Names = [Name || {_,Name} <- Progs],
+	    Nums = lists:seq(1, tuple_size(Progs0)),
+	    SubMenu = lists:zip(Names, Nums) ++
+		[{"< "++?__(1,"Next")++" >",next},
+		 {"< "++?__(2,"Previous")++">",prev}],
+	    [{?__(3,"Shaders: ")++shader_index(),
+	      {shader_set,SubMenu}}]
+    end.
+
+%% Only call this function if shaders are known to be supported.
+shader_index() ->
+    case wings_pref:get_value(number_of_lights) of
+	2 ->
+	    Progs = get(light_shaders),
+	    NumShaders = wings_pref:get_value(number_of_shaders),
+	    {_Prog,Name} = element(NumShaders, Progs),
+	    Name;
+	_ ->
+	    ""
+    end.
 
 along(x) -> along(x, -90.0, 0.0);
 along(y) -> along(y, 0.0, 90.0);
@@ -1152,7 +1244,8 @@ align_to_selection(#st{selmode=vertex}=St) ->
 				[wings_vertex:normal(V, We)|A]
 			end, Acc, Vs)
 	  end, St),
-    align_to_selection(N, St);
+	align_view_to_selection(N),
+	St;
 align_to_selection(#st{selmode=edge}=St) ->
     N = average_normals(
 	  fun(Edges, #we{es=Etab}=We, Acc) ->
@@ -1162,7 +1255,8 @@ align_to_selection(#st{selmode=edge}=St) ->
 				 wings_face:normal(Rf, We)|A]
 			end, Acc, Edges)
 	  end, St),
-    align_to_selection(N, St);
+	align_view_to_selection(N),
+	St;
 align_to_selection(#st{selmode=face}=St) ->
     N = average_normals(
 	  fun(Faces, We, Acc) ->
@@ -1170,9 +1264,15 @@ align_to_selection(#st{selmode=face}=St) ->
 				[wings_face:normal(Face, We)|A]
 			end, Acc, Faces)
 	  end, St),
-    align_to_selection(N, St);
+	align_view_to_selection(N),
+	St;
 align_to_selection(St) -> St.
 
+align_view_to_selection(N) ->
+    {Az,El} = align_view_to_normal(N),
+    View = current(),
+    set_current(View#view{azimuth=Az,elevation=El}).
+
 average_normals(CalcNormals, St) ->
     Ns = wings_sel:fold(
 	   fun(Items, We, Acc) ->
@@ -1180,7 +1280,7 @@ average_normals(CalcNormals, St) ->
 	   end, [], St),
     e3d_vec:norm(e3d_vec:add(Ns)).
 
-align_to_selection({Nx,Ny,Nz}, St) ->
+align_view_to_normal({Nx,Ny,Nz}) ->
     Z = {0.0,0.0,1.0},
     Az0 = e3d_vec:dot(e3d_vec:norm({Nx,0.0,Nz}), Z),
     Az1 = to_degrees(math:acos(Az0)),
@@ -1189,9 +1289,7 @@ align_to_selection({Nx,Ny,Nz}, St) ->
 	     true -> -Az1
 	 end,
     El = to_degrees(math:asin(Ny)),
-    View = current(),
-    set_current(View#view{azimuth=Az,elevation=El}),
-    St.
+	{Az,El}.
 
 to_degrees(A) when is_float(A) ->
     A*180.0/math:pi().
@@ -1332,8 +1430,6 @@ pos_legend({X,Y,Z}) ->
 	 true	 -> $+ end,Char]
      || {Abs,Char,Val} <- L, Abs >= Max*0.4142135624]. % tan(22.5 degrees)
 
-
-
 help_button(Subject) ->
     Title = help(title, Subject),
     TextFun = fun () -> help(text, Subject) end,
