@@ -5,12 +5,12 @@
 %%     we records (winged-edged records, the central data structure
 %%     in Wings 3D).
 %%
-%%  Copyright (c) 2001-2007 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_we.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_we).
@@ -35,8 +35,7 @@
 
 -include("wings.hrl").
 -include("e3d.hrl").
--import(lists, [map/2,foreach/2,foldl/3,sort/1,keysort/2,
-		last/1,reverse/1,duplicate/2,seq/2,filter/2,zip/2]).
+-import(lists, [foreach/2,foldl/3,sort/1,keysort/2,reverse/1,zip/2]).
 
 %%%
 %%% API.
@@ -183,7 +182,7 @@ hide_faces_1(Fs, #we{es=Etab0}=We0) ->
 		      R -> R
 		  end
 	  end,
-    Etab = wings_util:gb_trees_map(Map, Etab0),
+    Etab = gb_trees:map(Map, Etab0),
     We = We0#we{es=Etab,fs=undefined},
     wings_facemat:hide_faces(rebuild(We)).
 
@@ -304,7 +303,7 @@ show_faces_1(#we{es=Etab0}=We0) ->
 		  R#edge{lf=Lf,rf=Rf};
 	     (_, R) -> R
 	  end,
-    Etab = wings_util:gb_trees_map(Map, Etab0),
+    Etab = gb_trees:map(Map, Etab0),
     We = We0#we{es=Etab,fs=undefined},
     wings_facemat:show_faces(rebuild(We)).
 
@@ -590,8 +589,8 @@ merge(We0, We1) ->
 merge([]) -> [];
 merge([We]) -> We;
 merge([#we{id=Id,name=Name}|_]=Wes0) ->
-    Pst  = merge_plugins(Wes0),
     Wes1 = merge_renumber(Wes0),
+    Pst  = merge_plugins(Wes1),
     MatTab = wings_facemat:merge(Wes1),
     {Vpt0,Et0,Ht0} = merge_1(Wes1),
     Vpt = gb_trees:from_orddict(Vpt0),
@@ -671,7 +670,7 @@ do_renumber(We0, Id) ->
     We.
 
 do_renumber(#we{mode=Mode,vp=Vtab0,es=Etab0,fs=Ftab0,
-		mat=MatTab0,he=Htab0,perm=Perm0,mirror=Mirror0}=We0,
+		mat=MatTab0,he=Htab0,perm=Perm0,mirror=Mirror0,pst=Pst0}=We0,
 	    Id, RootSet0) ->
     Vtab1 = gb_trees:to_list(Vtab0),
     Vmap = make_map(Vtab1, Id),
@@ -707,10 +706,25 @@ do_renumber(#we{mode=Mode,vp=Vtab0,es=Etab0,fs=Ftab0,
 		 true -> gb_trees:get(Mirror0, Fmap)
 	     end,
 
+    Pst_Elements = wings_plugin:check_plugins(save,Pst0),
+    Pst = foldl(fun
+        ({_,{Plugin,{vs,VsSet}}}, Pst1) ->
+            Vs = gb_sets:to_list(VsSet),
+            renum_elements_in_pst(vs,Plugin,Vs,Vmap,Pst1);
+        ({_,{Plugin,{es,EsSet}}}, Pst1) ->
+            Es = gb_sets:to_list(EsSet),
+            renum_elements_in_pst(es,Plugin,Es,Emap,Pst1);
+        ({_,{Plugin,{fs,FsSet}}}, Pst1) ->
+            Fs = gb_sets:to_list(FsSet),
+            renum_elements_in_pst(fs,Plugin,Fs,Fmap,Pst1);
+        ({remove,Plugin}, Pst1) ->
+            gb_trees:delete(Plugin,Pst1)
+        end, Pst0, Pst_Elements),
+
     RootSet = map_rootset(RootSet0, Emap, Vmap, Fmap),
     We = We0#we{mode=Mode,vc=undefined,fs=undefined,
 		vp=Vtab,es=Etab,mat=MatTab,he=Htab,
-		perm=Perm,mirror=Mirror},
+		perm=Perm,mirror=Mirror,pst=Pst},
 
     %% In case this function will be used for merging #we records,
     %% it is essential to update the next_id field. Its value can
@@ -773,6 +787,22 @@ renum_hard_edge(Edge0, Emap, New) ->
     Edge = gb_trees:get(Edge0, Emap),
     [Edge|New].
 
+
+% Checks plugins which store elements in the Pst that need to be renumbered
+% before saving and returns the new Pst.
+renum_elements_in_pst(Key,Plugin,Elements,Map,Pst0) ->
+    Renumbered = foldl(fun(Elem, New) ->
+      case gb_trees:lookup(Elem,Map) of
+          none -> New;
+          {_,NewNum} -> [NewNum|New]
+      end
+    end, [], Elements),
+    Data = gb_trees:get(Plugin,Pst0),
+    NewElems = gb_sets:from_list(Renumbered),
+    NewData = gb_trees:update(Key,NewElems,Data),
+    Pst = gb_trees:update(Plugin,NewData,Pst0),
+    Pst.
+
 update_id_bounds(#we{vp=Vtab,es=Etab,fs=Ftab}=We) ->
     case gb_trees:is_empty(Etab) of
 	true -> We#we{next_id=0};
